"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AndroidPluginConfigGenerator = void 0;
const path_1 = __importDefault(require("path"));
const fs_readdir_recursive_1 = __importDefault(require("fs-readdir-recursive"));
const parser_1 = __importDefault(require("gradle-to-js/lib/parser"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const mustache_1 = __importDefault(require("mustache"));
const log_1 = __importDefault(require("./log"));
const template = `package com.walmartlabs.ern.container.plugins;

import android.app.Application;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.facebook.react.ReactPackage;
import {{packageName}}.{{className}};

public class {{className}}Plugin implements ReactPlugin {
    @Override
    public ReactPackage hook(@NonNull Application application, @Nullable ReactPluginConfig config) {
        return new {{className}}();
    }
}
`;
class AndroidPluginConfigGenerator {
    static fromPath(p) {
        return new AndroidPluginConfigGenerator(p);
    }
    constructor(p) {
        this.excludedDirectoriesRe = new RegExp(/sample|demo|example/i);
        this.exclusions = [
            'com.facebook.react:react-native',
            'fileTree(',
        ];
        this.root = p;
        this.files = (0, fs_readdir_recursive_1.default)(p, (x) => !this.excludedDirectoriesRe.test(x));
    }
    async generateConfig({ revolveBuildGradlePath, resolveDependencyVersion, }) {
        if (!this.doesPluginSupportAndroid) {
            throw new Error('No Android project found');
        }
        const config = {};
        let buildGradlePath = this.gradlePaths[0];
        if (this.gradlePaths.length > 1) {
            if (this.androidManifestPaths.length > 1) {
                buildGradlePath = await revolveBuildGradlePath(this.gradlePaths);
            }
            else {
                const manifestPath = this.androidManifestPaths[0];
                // find build.gradle path closest to AndroidManifest.xml
                let p = path_1.default.dirname(manifestPath);
                while (p !== '.') {
                    if (this.gradlePaths.map(path_1.default.dirname).includes(p)) {
                        buildGradlePath = path_1.default.join(p, 'build.gradle');
                        break;
                    }
                    p = path_1.default.dirname(p);
                }
            }
        }
        config.root = path_1.default.dirname(buildGradlePath);
        config.dependencies = await this.getDependenciesFromBuildGradle(path_1.default.join(this.root, buildGradlePath), resolveDependencyVersion);
        return config;
    }
    get doesPluginSupportAndroid() {
        return this.gradlePaths.length > 0;
    }
    async getDependenciesFromBuildGradle(p, resolveDependencyVersion) {
        const parsed = await parser_1.default.parseFile(p);
        // A bug in g2js can result in issues parsing the dependencies block
        // Details: https://github.com/ninetwozero/gradle-to-js/issues/32
        // TODO: Remove after the issue has been fixed in gradle-to-js
        if (!parsed.dependencies && parsed.buildscript) {
            parsed.dependencies = parsed.buildscript.dependencies;
        }
        const res = await Promise.all(parsed.dependencies
            .filter((x) => !/testImplementation|testCompile/.test(x.type))
            .filter((x) => !this.exclusions.includes(`${x.group}:${x.name}`))
            .filter((x) => !this.exclusions.some((y) => x.name.includes(y)))
            .map(async (x) => {
            if (x.group) {
                return `${x.type} '${x.group}:${x.name}:${x.version}'`;
            }
            else {
                // Handle this kind of string where version is dynamically retrieved
                // "com.android.support:exifinterface:${safeExtGet('supportLibVersion', '28.0.0')}"
                const match = x.name.replace('"', '').match(/^(.+):(.+):(.+)/);
                if (match) {
                    const [, group, name, version] = match;
                    const versionMatch = version.match(/\d+.\d+.\d+/);
                    if (versionMatch) {
                        return `${x.type} '${group}:${name}:${versionMatch[0]}'`;
                    }
                    else {
                        // Handle this kind of string where version cannot be found inline
                        // com.google.android.gms:play-services-vision:$googlePlayServicesVisionVersion
                        const resolvedVersion = await resolveDependencyVersion(`${group}:${name}`);
                        return `${x.type} '${group}:${name}:${resolvedVersion}'`;
                    }
                }
                log_1.default.debug(`Ignoring ${x.name}`);
                return '';
            }
        }));
        return res.filter((x) => x !== '');
    }
    async generatePluginSource() {
        const impls = await this.findReactPackageImplementations();
        if (impls.length === 0) {
            throw new Error('ReactPackage implementation not found');
        }
        if (impls.length > 1) {
            log_1.default.warn('Multiple ReactPackage impls detected. Manually verify the generated plugin and make modifications as necessary.');
        }
        log_1.default.info(`Using ReactPackage from: ${impls[0].file}`);
        log_1.default.debug(JSON.stringify(impls[0], null, 2));
        return {
            content: mustache_1.default.render(template, impls[0]),
            filename: `${impls[0].className}Plugin.java`,
        };
    }
    async findReactPackageImplementations() {
        const results = [];
        let files = this.files.filter((x) => /Package.(java|kt)$/.test(x));
        if (files.length === 0) {
            files = this.files.filter((x) => /.(java|kt)$/.test(x));
        }
        for (const file of files) {
            const content = await fs_extra_1.default.readFile(path_1.default.join(this.root, file));
            const source = content.toString();
            const packageName = this.getPackageDeclarationFromSource(source);
            const className = this.getClassNameFromSource(source);
            if (packageName && className) {
                results.push({ file, packageName, className });
                if (this.hasCustomConstructors(source, className)) {
                    log_1.default.info(`Custom constructors found in ${className}.`);
                    if (this.hasNoArgumentConstructor(source, className)) {
                        log_1.default.warn('It might be necessary to manually update the plugin config to make it a configurable plugin.');
                    }
                    else {
                        log_1.default.warn('Manual updates are required in generated plugin for packages without a no-argument constructor.');
                    }
                }
            }
        }
        return results;
    }
    getPackageDeclarationFromSource(source) {
        const match = source.match(/package ([\w.]+);?/);
        return match ? match[1] : null;
    }
    getClassNameFromSource(source) {
        const match = source.match(/class\s+(\w+)\(?.*\)?\s*(implements|:)\s*ReactPackage/);
        return match ? match[1] : null;
    }
    hasNoArgumentConstructor(source, className) {
        const re = new RegExp(`(public ${className}|constructor)\\((\\t*|\\s*)\\)`, 'g');
        return re.test(source);
    }
    hasCustomConstructors(source, className) {
        const re = new RegExp(`((public|class) ${className}|constructor)\\s*\\([^\\s).]+`, 'g');
        const match = source.match(re);
        return match !== null && match.length > 0;
    }
    get gradlePaths() {
        return this.files.filter((x) => x.endsWith('build.gradle'));
    }
    get androidManifestPaths() {
        return this.files.filter((x) => x.endsWith('AndroidManifest.xml'));
    }
}
exports.AndroidPluginConfigGenerator = AndroidPluginConfigGenerator;
//# sourceMappingURL=AndroidPluginConfigGenerator.js.map