"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BundleStoreEngine = void 0;
const BundleStoreSdk_1 = require("./BundleStoreSdk");
const config_1 = __importDefault(require("./config"));
const createTmpDir_1 = __importDefault(require("./createTmpDir"));
const log_1 = __importDefault(require("./log"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const got_1 = __importDefault(require("got"));
const node_ipc_1 = __importDefault(require("node-ipc"));
const yazl_1 = __importDefault(require("yazl"));
const createProxyAgent_1 = require("./createProxyAgent");
class BundleStoreEngine {
    constructor(host) {
        this.host = host;
        this.sdk = new BundleStoreSdk_1.BundleStoreSdk(host);
        this.assets = [];
        node_ipc_1.default.config.silent = true;
        node_ipc_1.default.config.id = 'ern-bundle-store';
        node_ipc_1.default.serve(() => {
            node_ipc_1.default.server.on('assets', (data, socket) => {
                log_1.default.debug(`received asset : ${JSON.stringify(data)}`);
                this.assets.push(JSON.parse(data));
            });
        });
        node_ipc_1.default.server.start();
    }
    async upload({ bundlePath, platform, sourceMapPath, }) {
        const res = await this.sdk.uploadBundle({
            accessKey: config_1.default.get('bundlestore-accesskey'),
            bundlePath,
            platform,
            sourceMapPath,
            store: config_1.default.get('bundlestore-id'),
        });
        await this.uploadAssets();
        node_ipc_1.default.server.stop();
        return res;
    }
    async uploadAssets() {
        const newAssets = await this.sdk.assetsDelta(this.assets.map((a) => a.hash));
        if (newAssets.length > 0) {
            log_1.default.debug(`Uploading ${newAssets.length} new asset(s)`);
            const zipfile = new yazl_1.default.ZipFile();
            const tmpZipPath = path_1.default.join((0, createTmpDir_1.default)(), 'assets.zip');
            const s = fs_1.default.createWriteStream(tmpZipPath);
            zipfile.outputStream.pipe(s);
            for (const asset of this.assets) {
                for (const file of asset.files) {
                    const hash = asset.hash;
                    zipfile.addFile(file, path_1.default.join(hash, path_1.default.basename(file)));
                }
            }
            zipfile.end();
            const assetsProm = new Promise((resolve, reject) => {
                s.on('close', () => {
                    resolve();
                });
            });
            await assetsProm;
            await this.sdk.uploadAssets(tmpZipPath);
        }
        else {
            log_1.default.debug('No new assets !');
        }
    }
    async uploadFromPackager({ dev, platform, } = {}) {
        const tmpDir = (0, createTmpDir_1.default)();
        const bundlePath = path_1.default.join(tmpDir, 'index.bundle');
        const sourceMapPath = path_1.default.join(tmpDir, 'index.map');
        const streamBundle = fs_1.default.createWriteStream(bundlePath);
        const streamSourceMap = fs_1.default.createWriteStream(sourceMapPath);
        const ernAgent = (0, createProxyAgent_1.createProxyAgentFromErnConfig)('bundleStoreProxy');
        const agent = {};
        if (ernAgent === null || ernAgent === void 0 ? void 0 : ernAgent.isHttpsAgent) {
            agent.https = ernAgent.agent;
        }
        else {
            agent.http = ernAgent === null || ernAgent === void 0 ? void 0 : ernAgent.agent;
        }
        const gotOptions = { agent };
        const reqBundle = got_1.default.stream(`http://localhost:8081/index.bundle?platform=${platform}&dev=${!!dev}&minify=true`, gotOptions);
        const reqSourceMap = got_1.default.stream(`http://localhost:8081/index.map?platform=${platform}&dev=${!!dev}&minify=true`, gotOptions);
        const sBundle = reqBundle.pipe(streamBundle);
        const sSourceMap = reqSourceMap.pipe(streamSourceMap);
        const pBundle = new Promise((resolve, reject) => sBundle.on('finish', () => resolve()));
        const pSourceMap = new Promise((resolve, reject) => {
            sSourceMap.on('finish', () => resolve());
        });
        await Promise.all([pBundle, pSourceMap]);
        const res = await this.sdk.uploadBundle({
            accessKey: config_1.default.get('bundlestore-accesskey'),
            bundlePath,
            platform: 'android',
            sourceMapPath,
            store: config_1.default.get('bundlestore-id'),
        });
        streamBundle.close();
        streamSourceMap.close();
        await this.uploadAssets();
        node_ipc_1.default.server.stop();
        return res;
    }
}
exports.BundleStoreEngine = BundleStoreEngine;
//# sourceMappingURL=BundleStoreEngine.js.map