"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ErnBinaryStore = void 0;
const createTmpDir_1 = __importDefault(require("./createTmpDir"));
const shell_1 = __importDefault(require("./shell"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const archiver_1 = __importDefault(require("archiver"));
const got_1 = __importDefault(require("got"));
const form_data_1 = __importDefault(require("form-data"));
const getGotCommonOpts_1 = require("./getGotCommonOpts");
// tslint:disable-next-line:no-var-requires
const AdmZip = require('adm-zip');
class ErnBinaryStore {
    constructor(config) {
        this.config = config;
    }
    async addBinary(descriptor, binaryPath, { flavor, } = {}) {
        var _a, _b;
        const pathToBinary = await this.zipBinary(descriptor, binaryPath, {
            flavor,
        });
        try {
            const binaryRs = fs_extra_1.default.createReadStream(pathToBinary);
            const form = new form_data_1.default();
            form.append('file', binaryRs);
            await got_1.default.post(this.config.url, { ...(0, getGotCommonOpts_1.getGotCommonOpts)(), body: form });
        }
        catch (err) {
            throw new Error((_b = (_a = err.response) === null || _a === void 0 ? void 0 : _a.text) !== null && _b !== void 0 ? _b : err.message);
        }
    }
    async removeBinary(descriptor, { flavor, } = {}) {
        var _a, _b;
        await this.throwIfNoBinaryExistForDescriptor(descriptor, { flavor });
        try {
            await got_1.default.delete(this.urlToBinary(descriptor, { flavor }), {
                ...(0, getGotCommonOpts_1.getGotCommonOpts)(),
            });
        }
        catch (err) {
            throw new Error((_b = (_a = err.response) === null || _a === void 0 ? void 0 : _a.text) !== null && _b !== void 0 ? _b : err.message);
        }
    }
    async getBinary(descriptor, { flavor, outDir, } = {}) {
        await this.throwIfNoBinaryExistForDescriptor(descriptor, { flavor });
        const pathToZippedBinary = await this.getZippedBinary(descriptor, {
            flavor,
        });
        if (outDir) {
            await fs_extra_1.default.ensureDir(outDir);
        }
        return this.unzipBinary(descriptor, pathToZippedBinary, { flavor, outDir });
    }
    async hasBinary(descriptor, { flavor, } = {}) {
        var _a, _b;
        try {
            const res = await got_1.default.head(this.urlToBinary(descriptor, { flavor }), {
                ...(0, getGotCommonOpts_1.getGotCommonOpts)(),
            });
            return res.statusCode === 200;
        }
        catch (err) {
            if (err.response && err.response.statusCode === 404) {
                return false;
            }
            throw new Error((_b = (_a = err.response) === null || _a === void 0 ? void 0 : _a.text) !== null && _b !== void 0 ? _b : err.message);
        }
    }
    async getZippedBinary(descriptor, { flavor, } = {}) {
        return new Promise((resolve, reject) => {
            const tmpOutDir = (0, createTmpDir_1.default)();
            const outputFilePath = path_1.default.join(tmpOutDir, this.buildZipBinaryFileName(descriptor, { flavor }));
            const outputFile = fs_extra_1.default.createWriteStream(outputFilePath);
            const gotStream = got_1.default
                .stream(this.urlToBinary(descriptor, { flavor }))
                .pipe(outputFile);
            gotStream.on('close', () => resolve(outputFilePath));
            gotStream.on('error', (err) => reject(err));
        });
    }
    urlToBinary(descriptor, { flavor, } = {}) {
        return `${this.config.url}/${this.buildZipBinaryFileName(descriptor, {
            flavor,
        })}`;
    }
    async zipBinary(descriptor, binaryPath, { flavor, } = {}) {
        return new Promise((resolve, reject) => {
            const tmpOutDir = (0, createTmpDir_1.default)();
            const pathToZipFile = path_1.default.join(tmpOutDir, this.buildZipBinaryFileName(descriptor, { flavor }));
            const outputZipStream = fs_extra_1.default.createWriteStream(pathToZipFile);
            const archive = (0, archiver_1.default)('zip', { zlib: { level: 9 } });
            outputZipStream.on('close', () => resolve(pathToZipFile));
            archive.on('error', (err) => reject(err));
            archive.pipe(outputZipStream);
            if (descriptor.platform === 'android') {
                archive.file(binaryPath.toString(), {
                    name: path_1.default.basename(binaryPath.toString()),
                });
            }
            else {
                archive.glob('**/*', { cwd: binaryPath.toString() });
            }
            archive.finalize();
        });
    }
    async unzipBinary(descriptor, zippedBinaryPath, { flavor, outDir, } = {}) {
        return new Promise((resolve) => {
            const outputDirectory = outDir || (0, createTmpDir_1.default)();
            const pathToOutputBinary = path_1.default.join(outputDirectory, this.buildNativeBinaryFileName(descriptor, { flavor }));
            const zip = new AdmZip(zippedBinaryPath);
            if (descriptor.platform === 'android') {
                zip.extractAllTo(outputDirectory);
            }
            else {
                zip.extractAllTo(pathToOutputBinary);
            }
            if (descriptor.platform === 'android') {
                shell_1.default.mv(path_1.default.join(outputDirectory, '*.apk'), pathToOutputBinary);
            }
            resolve(pathToOutputBinary);
        });
    }
    buildZipBinaryFileName(descriptor, { flavor, } = {}) {
        return `${descriptor.name}-${descriptor.platform}-${descriptor.version}${flavor ? `-${flavor}` : ''}.zip`;
    }
    buildNativeBinaryFileName(descriptor, { flavor, } = {}) {
        return `${descriptor.name}-${descriptor.platform}-${descriptor.version}${flavor ? `-${flavor}` : ''}.${this.getNativeBinaryFileExt(descriptor.platform)}`;
    }
    getNativeBinaryFileExt(platformName) {
        return platformName === 'android' ? 'apk' : 'app';
    }
    async throwIfNoBinaryExistForDescriptor(descriptor, { flavor, } = {}) {
        if (!(await this.hasBinary(descriptor, { flavor }))) {
            throw new Error(`No binary associated to ${descriptor} ${flavor ? `[flavor: ${flavor}]` : ''} was found in the store`);
        }
    }
}
exports.ErnBinaryStore = ErnBinaryStore;
//# sourceMappingURL=ErnBinaryStore.js.map