export interface CacheManifest {
    /**
     * Entries stored in the cache
     */
    entries: CacheManifestEntry[];
    /**
     * Total size of the cache
     */
    size: number;
}
export interface CacheManifestEntry {
    /**
     * Id of the entry
     * ex: my-miniapp@1.0.0
     */
    id: string;
    /**
     * Absolute path to the entry
     * ex : /home/user/.ern/cache/78c40fda-d5bb-4681-9bd2-a5fba9df1f70
     */
    path: string;
    /**
     * Size in bytes of the entry
     */
    size: number;
    /**
     * Time (epoc) when the entry was last accessed
     */
    lastAccessed: number;
}
export declare const maxDefaultCacheSize: number;
type AddObjectToCacheDirectory<T> = (obj: T, dirPath: string) => Promise<void>;
type ObjectToId<T> = (obj: T) => string;
/**
 * Simple generic file system based LRU cache.
 * This cache stores arbirtrary objects in unique directories.
 * The root directory for the cache can be configured, as well as
 * the maximum size allowed for the cache.
 * Once cache reaches max size, the least recenctly accessed
 * objects will be removed from the cache, until size of the cache
 * falls below maximum size.
 */
export declare class FsCache<T> {
    /**
     * Root path to the directory holding the cache
     */
    readonly rootCachePath: string;
    /**
     * Maximum cache size (in bytes)
     */
    readonly maxCacheSize: number;
    /**
     * Custom function invoked to add an object entry to the cache
     */
    private readonly addObjectToCacheDirectory;
    /**
     * Custom function invoked to convert an object entry to a unique id
     */
    private readonly objectToId;
    constructor({ 
    /**
     * Custom function invoked to add an object entry to the cache
     */
    addObjectToCacheDirectory, 
    /**
     * Custom function invoked to convert an object entry to a unique id
     */
    objectToId, 
    /**
     * Maximum cache size (in bytes)
     */
    maxCacheSize, 
    /**
     * Root path to the directory holding the cache
     */
    rootCachePath, }: {
        addObjectToCacheDirectory: AddObjectToCacheDirectory<T>;
        objectToId: ObjectToId<T>;
        maxCacheSize?: number;
        rootCachePath: string;
    });
    /**
     * Indicates whether a given object is currently stored in the cache
     * @param obj The object to check for presence in the cache
     */
    isInCache(obj: T): Promise<boolean>;
    /**
     * Adds an object entry to the cache
     * @param obj The object to add to the cache
     */
    addToCache(obj: T): Promise<string>;
    /**
     * Gets path to the directory containing a cached object
     * @param obj The object for which to retrieve cached directory path
     */
    getObjectCachePath(obj: T): Promise<string | void>;
    /**
     * Gets the cache manifest object
     */
    private getManifestObj;
    /**
     * Creates a uniquely named cache directory
     */
    private createUniqueCacheDirectory;
    /**
     * Gets the path to the cache manifest file
     */
    private get cacheManifestPath();
}
export {};
