"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FsCache = exports.maxDefaultCacheSize = void 0;
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const shell_1 = __importDefault(require("./shell"));
const uuid_1 = require("uuid");
const get_folder_size_1 = __importDefault(require("get-folder-size"));
/**
 * Gets the total size (in bytes) of a given directory
 * @param dirPath path to the directory for which to compute size
 */
const getSizeAsync = (dirPath) => new Promise((resolve, reject) => {
    (0, get_folder_size_1.default)(dirPath, (err, size) => {
        err ? reject(err) : resolve(size);
    });
});
const cacheManifestFileName = 'cache-manifest.json';
exports.maxDefaultCacheSize = 2 * 1024 * 1024 * 1024; /* 2GB */
/**
 * Simple generic file system based LRU cache.
 * This cache stores arbirtrary objects in unique directories.
 * The root directory for the cache can be configured, as well as
 * the maximum size allowed for the cache.
 * Once cache reaches max size, the least recenctly accessed
 * objects will be removed from the cache, until size of the cache
 * falls below maximum size.
 */
class FsCache {
    constructor({ 
    /**
     * Custom function invoked to add an object entry to the cache
     */
    addObjectToCacheDirectory, 
    /**
     * Custom function invoked to convert an object entry to a unique id
     */
    objectToId, 
    /**
     * Maximum cache size (in bytes)
     */
    maxCacheSize = exports.maxDefaultCacheSize, 
    /**
     * Root path to the directory holding the cache
     */
    rootCachePath, }) {
        this.rootCachePath = rootCachePath;
        this.maxCacheSize = maxCacheSize;
        this.addObjectToCacheDirectory = addObjectToCacheDirectory;
        this.objectToId = objectToId;
        fs_extra_1.default.ensureDirSync(rootCachePath);
        if (!fs_extra_1.default.existsSync(this.cacheManifestPath)) {
            fs_extra_1.default.writeJsonSync(this.cacheManifestPath, { entries: [], size: 0 }, { spaces: 2 });
        }
    }
    /**
     * Indicates whether a given object is currently stored in the cache
     * @param obj The object to check for presence in the cache
     */
    async isInCache(obj) {
        const manifestObj = await this.getManifestObj();
        return !!manifestObj.entries.find((e) => e.id === this.objectToId(obj));
    }
    /**
     * Adds an object entry to the cache
     * @param obj The object to add to the cache
     */
    async addToCache(obj) {
        if (await this.isInCache(obj)) {
            throw new Error(`Object with id ${this.objectToId(obj)} is already in the cache`);
        }
        // Create unique directory to store the object
        const pathToObjectCacheDir = this.createUniqueCacheDirectory();
        // Invoke client provided function to write object in cache directory
        await this.addObjectToCacheDirectory(obj, pathToObjectCacheDir);
        // Compute cached object directory size
        const cachedObjectDirSize = await getSizeAsync(pathToObjectCacheDir);
        // Create manifest entry for new cached object
        const entry = {
            id: this.objectToId(obj),
            lastAccessed: Date.now(),
            path: pathToObjectCacheDir,
            size: cachedObjectDirSize,
        };
        // Update manifest total cache size
        const manifestObj = await this.getManifestObj();
        manifestObj.size += cachedObjectDirSize;
        manifestObj.entries.push(entry);
        // Clean cache if newly added object caused total cache
        // size to exceed the defined max cache size
        // Remove all entries that were least recently accessed
        // until the total cache size falls below the max cache size
        const entriesSortedByLat = manifestObj.entries.sort((a, b) => b.lastAccessed - a.lastAccessed);
        while (manifestObj.size >= this.maxCacheSize) {
            const entryToRemove = entriesSortedByLat.pop();
            manifestObj.size -= entryToRemove.size;
            shell_1.default.rm('-rf', entryToRemove.id);
        }
        manifestObj.entries = entriesSortedByLat;
        // Save updated manifest
        await fs_extra_1.default.writeJson(this.cacheManifestPath, manifestObj, { spaces: 2 });
        return pathToObjectCacheDir;
    }
    /**
     * Gets path to the directory containing a cached object
     * @param obj The object for which to retrieve cached directory path
     */
    async getObjectCachePath(obj) {
        const manifestObj = await this.getManifestObj();
        const cacheEntry = manifestObj.entries.find((e) => e.id === this.objectToId(obj));
        if (cacheEntry) {
            cacheEntry.lastAccessed = Date.now();
            await fs_extra_1.default.writeJson(this.cacheManifestPath, manifestObj, { spaces: 2 });
            return cacheEntry.path;
        }
    }
    /**
     * Gets the cache manifest object
     */
    async getManifestObj() {
        return fs_extra_1.default.readJson(this.cacheManifestPath);
    }
    /**
     * Creates a uniquely named cache directory
     */
    createUniqueCacheDirectory() {
        const pathToCacheDirectory = path_1.default.join(this.rootCachePath, (0, uuid_1.v4)());
        shell_1.default.mkdir(pathToCacheDirectory);
        return pathToCacheDirectory;
    }
    /**
     * Gets the path to the cache manifest file
     */
    get cacheManifestPath() {
        return path_1.default.join(this.rootCachePath, cacheManifestFileName);
    }
}
exports.FsCache = FsCache;
//# sourceMappingURL=FsCache.js.map