export declare class GitHubApi {
    private readonly octokit;
    private readonly owner;
    private readonly repo;
    /**
     * Creates an instance of the GitHubApi class
     *
     * opts: Ocktokit options
     * owner: Org or user owning the repository
     * repo: Name of the repository in which to create the branch
     */
    constructor({ opts, owner, repo, }: {
        opts: any;
        owner: string;
        repo: string;
    });
    /**
     * Creates a branch in a GitHub repository.
     * The branch can be created from another branch, sha or tag.
     * If none provided, the branch will be created from default branch.
     *
     * fromBranch: Name of the branch to create branch from
     * fromSha: SHA1 of the commit to create branch from
     * fromTag: Name of the tag to create branch from
     * name: Name of the branch to create
     */
    createBranch({ fromBranch, fromSha, fromTag, name, }: {
        fromBranch?: string;
        fromSha?: string;
        fromTag?: string;
        name: string;
    }): Promise<{
        headers: {
            [x: string]: string | number;
            "cache-control": string;
            "content-length": number;
            "content-type": string;
            date: string;
            etag: string;
            "last-modified": string;
            link: string;
            location: string;
            server: string;
            status: string;
            vary: string;
            "x-github-mediatype": string;
            "x-github-request-id": string;
            "x-oauth-scopes": string;
            "x-ratelimit-limit": string;
            "x-ratelimit-remaining": string;
            "x-ratelimit-reset": string;
        };
        status: 201;
        url: string;
        data: {
            ref: string;
            node_id: string;
            url: string;
            object: {
                type: string;
                sha: string;
                url: string;
            };
        };
    }>;
    /**
     * Deletes a branch from the GitHub repository
     *
     * name: Name of the branch to delete
     */
    deleteBranch({ name }: {
        name: string;
    }): Promise<import("@octokit/types").OctokitResponse<never, 204>>;
    /**
     * Creates a tag in a GitHub repository.
     * The tag can be created from another branch, sha or tag.
     * If none provided, the tag will be created from default branch.
     *
     * fromBranch: Name of the branch to create tag from
     * fromSha: SHA1 of the commit to create tag from
     * fromTag: Name of the tag to create tag from
     * name: Name of the tag to create
     */
    createTag({ fromBranch, fromSha, fromTag, name, }: {
        fromBranch?: string;
        fromSha?: string;
        fromTag?: string;
        name: string;
    }): Promise<{
        headers: {
            [x: string]: string | number;
            "cache-control": string;
            "content-length": number;
            "content-type": string;
            date: string;
            etag: string;
            "last-modified": string;
            link: string;
            location: string;
            server: string;
            status: string;
            vary: string;
            "x-github-mediatype": string;
            "x-github-request-id": string;
            "x-oauth-scopes": string;
            "x-ratelimit-limit": string;
            "x-ratelimit-remaining": string;
            "x-ratelimit-reset": string;
        };
        status: 201;
        url: string;
        data: {
            ref: string;
            node_id: string;
            url: string;
            object: {
                type: string;
                sha: string;
                url: string;
            };
        };
    }>;
    /**
     * Deletes a tag from the GitHub repository
     *
     * name: Name of the tag to delete
     */
    deleteTag({ name }: {
        name: string;
    }): Promise<import("@octokit/types").OctokitResponse<never, 204>>;
    /**
     * Retrieves the content of a given file from the repository
     *
     * path: Path to the file in the repository
     * fromBranch: Branch to retrieve the file from
     * fromTag: Tag to retrieve the file from
     *
     * If neither of fromBranch/fromTag are provided, the file
     * will be retrieved from the default repository branch
     */
    getFileContent({ path, fromBranch, fromTag, }: {
        path: string;
        fromBranch?: string;
        fromTag?: string;
    }): Promise<string>;
    /**
     * Updates the content of a given file in the repository
     *
     * path: Path to the file to update
     * onBranch: Name of the branch on which to update the file
     * newContent: New file content
     * commitMessage: Commit message for this update
     */
    updateFileContent({ path, onBranch, newContent, commitMessage, }: {
        path: string;
        onBranch?: string;
        newContent: string;
        commitMessage: string;
    }): Promise<{
        headers: {
            [x: string]: string | number;
            "cache-control": string;
            "content-length": number;
            "content-type": string;
            date: string;
            etag: string;
            "last-modified": string;
            link: string;
            location: string;
            server: string;
            status: string;
            vary: string;
            "x-github-mediatype": string;
            "x-github-request-id": string;
            "x-oauth-scopes": string;
            "x-ratelimit-limit": string;
            "x-ratelimit-remaining": string;
            "x-ratelimit-reset": string;
        };
        status: 200;
        url: string;
        data: {
            content: {
                name: string;
                path: string;
                sha: string;
                size: number;
                url: string;
                html_url: string;
                git_url: string;
                download_url: string;
                type: string;
                _links: {
                    self: string;
                    git: string;
                    html: string;
                };
            };
            commit: {
                sha: string;
                node_id: string;
                url: string;
                html_url: string;
                author: {
                    date: string;
                    name: string;
                    email: string;
                };
                committer: {
                    date: string;
                    name: string;
                    email: string;
                };
                message: string;
                tree: {
                    url: string;
                    sha: string;
                };
                parents: {
                    url: string;
                    html_url: string;
                    sha: string;
                }[];
                verification: {
                    verified: boolean;
                    reason: string;
                    signature: string;
                    payload: string;
                };
            };
        };
    } | {
        headers: {
            [x: string]: string | number;
            "cache-control": string;
            "content-length": number;
            "content-type": string;
            date: string;
            etag: string;
            "last-modified": string;
            link: string;
            location: string;
            server: string;
            status: string;
            vary: string;
            "x-github-mediatype": string;
            "x-github-request-id": string;
            "x-oauth-scopes": string;
            "x-ratelimit-limit": string;
            "x-ratelimit-remaining": string;
            "x-ratelimit-reset": string;
        };
        status: 201;
        url: string;
        data: {
            content: {
                name: string;
                path: string;
                sha: string;
                size: number;
                url: string;
                html_url: string;
                git_url: string;
                download_url: string;
                type: string;
                _links: {
                    self: string;
                    git: string;
                    html: string;
                };
            };
            commit: {
                sha: string;
                node_id: string;
                url: string;
                html_url: string;
                author: {
                    date: string;
                    name: string;
                    email: string;
                };
                committer: {
                    date: string;
                    name: string;
                    email: string;
                };
                message: string;
                tree: {
                    url: string;
                    sha: string;
                };
                parents: {
                    url: string;
                    html_url: string;
                    sha: string;
                }[];
                verification: {
                    verified: boolean;
                    reason: string;
                    signature: string;
                    payload: string;
                };
            };
        };
    }>;
    /**
     * Gets the SHA1 of a given branch or tag.
     * If no branch or tag is provided, the SHA1 of the default branch
     * will be returned.
     *
     * ofBranch: Name of the branch to get SHA1 of
     * ofTag: Name of the tag to get SHA1 of
     */
    getSha({ ofBranch, ofTag, }?: {
        ofBranch?: string;
        ofTag?: string;
    }): Promise<string>;
    /**
     * Gets the default branch name of the repository.
     */
    getDefaultBranch(): Promise<string>;
    /**
     * Does a branch name exist in the repository ?
     */
    isBranch(x: string): Promise<boolean>;
    /**
     * Does a tag name exist in the repository ?
     */
    isTag(x: string): Promise<boolean>;
    /**
     * Does a SHA exist in the repository ?
     */
    isSha(x: string): Promise<boolean>;
    fullTagRef(tag: string): string;
    shortTagRef(tag: string): string;
    fullBranchRef(branch: string): string;
    shortBranchRef(branch: string): string;
}
