"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const shell_1 = __importDefault(require("./shell"));
const gitCli_1 = require("./gitCli");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const Platform_1 = __importDefault(require("./Platform"));
const log_1 = __importDefault(require("./log"));
const kax_1 = __importDefault(require("./kax"));
const LocalManifest_1 = require("./LocalManifest");
class GitManifest extends LocalManifest_1.LocalManifest {
    constructor(repoLocalPath, repoRemotePath, { branch, remote } = {
        branch: 'master',
        remote: 'origin',
    }) {
        super(repoLocalPath);
        this.repoLocalPath = repoLocalPath;
        this.repoRemotePath = repoRemotePath;
        this.git = (0, gitCli_1.gitCli)(repoLocalPath);
        this.remote = remote;
        this.branch = branch;
        this.hasSynced = false;
    }
    async sync() {
        if (this.repoRemotePath) {
            log_1.default.debug(`[GitManifest] Syncing ${this.repoRemotePath}`);
        }
        if (!fs_extra_1.default.existsSync(path_1.default.join(this.repoLocalPath, '.git'))) {
            log_1.default.debug(`[GitManifest] Creating local repository in ${this.repoLocalPath}`);
            shell_1.default.mkdir('-p', this.repoLocalPath);
            await this.git.init();
            if (this.repoRemotePath) {
                await this.git.addRemote(this.remote, this.repoRemotePath);
            }
        }
        if (this.repoRemotePath) {
            await this.git.raw([
                'remote',
                'set-url',
                this.remote,
                this.repoRemotePath,
            ]);
            try {
                log_1.default.debug(`[GitManifest] Fetching from ${this.remote} master`);
                await this.git.fetch(this.remote, 'master');
            }
            catch (e) {
                if (e.message.includes(`Couldn't find remote ref master`)) {
                    throw new Error(`It looks like no remote Manifest repository exist at ${this.repoRemotePath}`);
                }
                else {
                    throw e;
                }
            }
            await this.git.reset(['--hard', `${this.remote}/master`]);
            await this.git.pull(this.remote, this.branch);
        }
    }
    /**
     * We only sync once during a whole "session"
     * (in our context : "an ern command exection")
     * This is done to speed up things as during a single command execution,
     * multiple manifest access can be performed.
     * If you need to access a `Manifest` in a different context, a long session,
     *  you might want to improve the code to act a bit smarter.
     */
    async syncIfNeeded() {
        if (!this.hasSynced) {
            await kax_1.default
                .task(`Syncing ${this.repoRemotePath || this.repoLocalPath} Manifest`)
                .run(this.sync());
            this.hasSynced = true;
        }
    }
    async getManifest() {
        await this.syncIfNeeded();
        return super.getManifest();
    }
    async hasManifestId(manifestId) {
        await this.syncIfNeeded();
        return super.hasManifestId(manifestId);
    }
    async getManifestData({ manifestId = 'default', platformVersion = Platform_1.default.currentVersion, } = {}) {
        await this.syncIfNeeded();
        return super.getManifestData({ manifestId, platformVersion });
    }
    async getPluginConfigurationPath(plugin, platformVersion = Platform_1.default.currentVersion) {
        await this.syncIfNeeded();
        return super.getPluginConfigurationPath(plugin, platformVersion);
    }
}
exports.default = GitManifest;
//# sourceMappingURL=GitManifest.js.map