"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HermesCli = void 0;
const createTmpDir_1 = __importDefault(require("./createTmpDir"));
const shell_1 = __importDefault(require("./shell"));
const clients_1 = require("./clients");
const PackagePath_1 = require("./PackagePath");
const childProcess_1 = require("./childProcess");
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const semver_1 = __importDefault(require("semver"));
class HermesCli {
    static async fromVersion(version) {
        this.workingDir = (0, createTmpDir_1.default)();
        shell_1.default.pushd(this.workingDir);
        try {
            await clients_1.yarn.init();
            await clients_1.yarn.add(PackagePath_1.PackagePath.fromString(`hermes-engine@${version}`));
            return new HermesCli(path_1.default.join(this.workingDir, HermesCli.hermesModulePath));
        }
        finally {
            shell_1.default.popd();
        }
    }
    static get hermesVersion() {
        const pJson = fs_extra_1.default.readJSONSync(path_1.default.join(this.workingDir, 'node_modules/hermes-engine/package.json'));
        return pJson.version;
    }
    static get hermesBinaryName() {
        return semver_1.default.lt(this.hermesVersion, '0.5.0') ? 'hermes' : 'hermesc';
    }
    static get hermesModulePath() {
        return path_1.default.normalize(`node_modules/hermes-engine/${HermesCli.platformDirectory}/${this.hermesBinaryName}`);
    }
    static get platformDirectory() {
        return process.platform === 'darwin'
            ? 'osx-bin'
            : /^win/.test(process.platform)
                ? 'win64-bin'
                : 'linux64-bin';
    }
    constructor(hermesPath) {
        this.hermesPath = hermesPath;
    }
    async compileDebugBundle({ jsBundlePath, outputBundlePath, }) {
        const hermesBundlePath = outputBundlePath || jsBundlePath;
        await this.run({
            args: ['-emit-binary', '-out', hermesBundlePath, jsBundlePath],
        });
        return {
            hermesBundlePath,
        };
    }
    async compileReleaseBundle({ bundleSourceMapPath, compositePath, jsBundlePath, outputBundlePath, }) {
        const hermesBundlePath = outputBundlePath || jsBundlePath;
        const hermesSourceMapPath = `${hermesBundlePath}.map`;
        const packagerSourceMapPath = bundleSourceMapPath &&
            path_1.default.join(path_1.default.dirname(bundleSourceMapPath), 'index.android.packager.map');
        if (bundleSourceMapPath) {
            // Rename the existing JS bundle sourcemap as index.android.packager.map
            shell_1.default.mv(bundleSourceMapPath, packagerSourceMapPath);
        }
        await this.run({
            args: [
                '-emit-binary',
                '-out',
                hermesBundlePath,
                jsBundlePath,
                '-O',
                '-output-source-map',
            ],
        });
        // Hermes CLI does not give control over the
        // location of generated source map. It just generates it in
        // the same directory as the hermes bundle, with the same name
        // as the bundle, with a new .map extension (index.android.map)
        // Move it to same location as the JS bundle one, but rename it
        // to index.android.compiler.map
        const compilerSourceMapPath = path_1.default.join(path_1.default.dirname(bundleSourceMapPath || hermesBundlePath), 'index.android.compiler.map');
        shell_1.default.mv(hermesSourceMapPath, compilerSourceMapPath);
        // Delete the compiler source map if its not needed
        // Note that it's not possible to not generate it in the first place
        // (i.e remove the --output-source-map option from command above)
        // as it would still generate the source map but inlined in the
        // hermes bundle making its size much bigger
        if (!bundleSourceMapPath) {
            shell_1.default.rm(compilerSourceMapPath);
        }
        if (bundleSourceMapPath) {
            // Compose both source maps to get final one, and write it to sourcemap path
            const pathToComposeScript = path_1.default.join(compositePath, 'node_modules/react-native/scripts/compose-source-maps.js');
            shell_1.default.exec(`node ${pathToComposeScript} ${packagerSourceMapPath} ${compilerSourceMapPath} -o ${bundleSourceMapPath}`);
        }
        return {
            compilerSourceMapPath: bundleSourceMapPath
                ? compilerSourceMapPath
                : undefined,
            hermesBundlePath,
            packagerSourceMapPath,
        };
    }
    async run({ args }) {
        return (0, childProcess_1.spawnp)(this.hermesPath, args);
    }
}
exports.HermesCli = HermesCli;
//# sourceMappingURL=HermesCli.js.map