"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LocalManifest = void 0;
const PackagePath_1 = require("./PackagePath");
const lodash_1 = __importDefault(require("lodash"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const semver_1 = __importDefault(require("semver"));
const Platform_1 = __importDefault(require("./Platform"));
const manifestFileName = 'manifest.json';
const pluginConfigFileName = 'config.json';
const ERN_VERSION_DIRECTORY_RE = /ern_v(.+)\+/;
class LocalManifest {
    constructor(manifestPath) {
        this.manifestPath = manifestPath;
        fs_extra_1.default.ensureDirSync(manifestPath);
    }
    async getManifest() {
        const pathToManifestJson = path_1.default.join(this.manifestPath, manifestFileName);
        return Promise.resolve(JSON.parse(fs_extra_1.default.readFileSync(pathToManifestJson, 'utf-8')));
    }
    async hasManifestId(manifestId) {
        const manifest = await this.getManifest();
        return !Array.isArray(manifest) && manifest[manifestId];
    }
    async getManifestData({ manifestId = 'default', platformVersion = Platform_1.default.currentVersion, } = {}) {
        const manifest = await this.getManifest();
        return Array.isArray(manifest)
            ? // Old manifest format [to be deprecated]
                lodash_1.default.find(manifest, (m) => semver_1.default.satisfies(platformVersion, m.platformVersion))
            : // New manifest format
                manifest[manifestId];
    }
    /**
     * Return an array containing all top level plugin configuration directories
     * are matching an Electrode Native version lower or equal than the specified
     * maxVersion.
     *
     * For example, given the following directories :
     *
     *  /home/user/.ern/ern-override-manifest/plugins/ern_v0.5.0+
     *  /home/user/.ern/ern-override-manifest/plugins/ern_v0.10.0+
     *  /home/user/.ern/ern-override-manifest/plugins/ern_v0.13.0+
     *
     * And maxVersion='0.10.0'
     *
     * The function would return :
     *
     *  [
     *   '/home/user/.ern/ern-override-manifest/plugins/ern_v0.5.0+',
     *   '/home/user/.ern/ern-override-manifest/plugins/ern_v0.10.0+'
     *  ]
     * @param maxVersion The upper bound Electrode Native version.
     *                   Only plugin configuration directories lower than this
     *                   version will be returned.
     */
    getPluginsConfigurationDirectories(maxVersion = Platform_1.default.currentVersion) {
        const pathToPluginsDirectory = path_1.default.join(this.manifestPath, 'plugins');
        return fs_extra_1.default.existsSync(pathToPluginsDirectory)
            ? (0, lodash_1.default)(fs_extra_1.default.readdirSync(pathToPluginsDirectory))
                .filter((d) => ERN_VERSION_DIRECTORY_RE.test(d) &&
                semver_1.default.lte(ERN_VERSION_DIRECTORY_RE.exec(d)[1], maxVersion))
                .map((d) => path_1.default.join(pathToPluginsDirectory, d))
                .value()
            : [];
    }
    /**
     * Given a fixed versioned plugin, return the local file system path
     * to the directory containing the plugin configuration files, or
     * undefined if no configuration matching this plugin version exists
     * in the Manifest.
     * @param plugin Fixed versioned plugin for which to retrieve configuration
     * @param platformVersion Platform version that is querying for
     *                        the plugin configuration.
     */
    async getPluginConfigurationPath(plugin, platformVersion = Platform_1.default.currentVersion) {
        if (!plugin.version) {
            throw new Error(`Plugin version is required to retrieve configuration. No version was specified for ${plugin}`);
        }
        else if (!semver_1.default.valid(plugin.version)) {
            throw new Error('Plugin version is required to be fixed and not ranged to retrieve configuration');
        }
        let result = await this.getPluginConfigurationPathNew(plugin, platformVersion);
        if (!result) {
            result = await this.getPluginConfigurationPathLegacy(plugin, platformVersion);
        }
        return result;
    }
    async getPluginConfigurationPathNew(plugin, platformVersion = Platform_1.default.currentVersion) {
        if (!plugin.version) {
            throw new Error(`Plugin ${PackagePath_1.PackagePath.toString()} does not have a version`);
        }
        const versionRe = /_v(.+)\+/;
        const scopeNameRe = /^(@.+)\/(.+)$/;
        const packageNameRe = /(.+)_v\d+\.\d+\.\d+\+/;
        // Top level plugin configuration directories ordered by descending
        // Electrode Native version
        // For example :
        // [
        //   '/home/user/.ern/ern-override-manifest/plugins/ern_v0.10.0+',
        //   '/home/user/.ern/ern-override-manifest/plugins/ern_v0.5.0+'
        // ]
        const orderedPluginsConfigurationDirectories = this.getPluginsConfigurationDirectories(platformVersion).sort((a, b) => semver_1.default.rcompare(versionRe.exec(a)[1], versionRe.exec(b)[1]));
        for (const pluginsConfigurationDirectory of orderedPluginsConfigurationDirectories) {
            let pluginScope;
            let pluginName;
            let basePluginPath;
            if (scopeNameRe.test(plugin.name)) {
                pluginScope = scopeNameRe.exec(plugin.name)[1];
                pluginName = scopeNameRe.exec(plugin.name)[2];
                basePluginPath = path_1.default.join(pluginsConfigurationDirectory, pluginScope);
            }
            else {
                pluginName = plugin.name;
                basePluginPath = pluginsConfigurationDirectory;
            }
            if (!fs_extra_1.default.existsSync(basePluginPath)) {
                continue;
            }
            const pluginConfigDirectories = fs_extra_1.default
                .readdirSync(basePluginPath)
                .filter((f) => {
                const p = packageNameRe.exec(f);
                return p ? p[1] === pluginName : false;
            });
            const pluginVersions = lodash_1.default.map(pluginConfigDirectories, (s) => versionRe.exec(s)[1]);
            const matchingVersion = lodash_1.default.find(pluginVersions.sort(semver_1.default.rcompare), (d) => semver_1.default.gte(plugin.version, d));
            if (matchingVersion) {
                let pluginConfigurationPath = '';
                pluginConfigurationPath = path_1.default.join(basePluginPath, `${pluginName}_v${matchingVersion}+`);
                if (fs_extra_1.default.existsSync(path_1.default.join(pluginConfigurationPath, pluginConfigFileName))) {
                    return pluginConfigurationPath;
                }
            }
        }
    }
    /**
     * Old way of getting plugin configuration path
     *  -- To be deprecated --
     */
    async getPluginConfigurationPathLegacy(plugin, platformVersion = Platform_1.default.currentVersion) {
        if (!plugin.version) {
            throw new Error(`Plugin ${PackagePath_1.PackagePath.toString()} does not have a version`);
        }
        const versionRe = /_v(.+)\+/;
        const orderedPluginsConfigurationDirectories = this.getPluginsConfigurationDirectories(platformVersion).sort((a, b) => semver_1.default.rcompare(versionRe.exec(a)[1], versionRe.exec(b)[1]));
        for (const pluginsConfigurationDirectory of orderedPluginsConfigurationDirectories) {
            // Directory names cannot contain '/', so, replaced by ':'
            const pluginScopeAndName = plugin.name.replace(/\//g, ':');
            const pluginVersions = lodash_1.default.map(fs_extra_1.default
                .readdirSync(pluginsConfigurationDirectory)
                .filter((f) => f.startsWith(pluginScopeAndName)), (s) => versionRe.exec(s)[1]);
            const matchingVersion = lodash_1.default.find(pluginVersions.sort(semver_1.default.rcompare), (d) => semver_1.default.gte(plugin.version, d));
            if (matchingVersion) {
                const pluginConfigurationPath = path_1.default.join(pluginsConfigurationDirectory, `${pluginScopeAndName}_v${matchingVersion}+`);
                if (fs_extra_1.default.existsSync(path_1.default.join(pluginConfigurationPath, pluginConfigFileName))) {
                    return pluginConfigurationPath;
                }
            }
        }
    }
}
exports.LocalManifest = LocalManifest;
//# sourceMappingURL=LocalManifest.js.map