import { PackagePath } from './PackagePath';
import GitManifest from './GitManifest';
import { LocalManifest } from './LocalManifest';
import { NativePlatform } from './NativePlatform';
export type PluginConfig<T extends NativePlatform> = T extends 'android' ? AndroidPluginConfig : IosPluginConfig;
/**
 * Represent the plugin structure as seen in the manifest
 * i.e the config.json structure
 */
export interface ManifestPluginConfig {
    android?: PluginConfig<'android'>;
    ios?: PluginConfig<'ios'>;
    origin?: PluginOrigin;
    path?: string;
}
/**
 * Represent a NPM registry package where a plugin source code is located
 */
export interface NpmPluginOrigin {
    /**
     * Identifies the plugin origin (npm)
     */
    type: 'npm';
    /**
     * Optional registry scope of the plugin
     */
    scope?: string;
    /**
     * Name of the plugin
     */
    name: string;
    /**
     * Version of the plugin
     */
    version?: string;
}
/**
 * Represent a Git repository  where a plugin source code is located
 */
export interface GitPluginOrigin {
    /**
     * Identifies the plugin origin (git)
     */
    type: 'git';
    /**
     * Url of the git repository
     */
    url: string;
    /**
     * Version of the plugin
     */
    version: string;
}
export type PluginOrigin = NpmPluginOrigin | GitPluginOrigin;
/**
 * Platform independent plugin configuration
 */
export interface CommonPluginConfig extends CommonPluginDirectives {
    /**
     * Relative path to the directory containing the plugin source code.
     * Will default to 'ios' or 'android' unless specified otherwise.
     */
    root: string;
    /**
     * Optional plugin hook
     */
    pluginHook?: PluginHook;
    /**
     * Location of the source code of this plugin
     */
    origin: PluginOrigin;
    /**
     * Local path to the directory containing the plugin configuration
     */
    path?: string;
}
/**
 * Platform independent plugin directives
 */
export interface CommonPluginDirectives {
    /**
     * Apply a git patch
     */
    applyPatch?: PluginApplyPatchDirective;
    /**
     * Array of copy directives.
     * Represents the file(s) to be copied from the plugin source code
     * to the Container.
     */
    copy?: PluginCopyDirective[];
    /**
     * Array of replace in file directives.
     * Represents string replacements in plugin files, before adding them
     * to the Container.
     */
    replaceInFile?: PluginReplaceInFileDirective[];
}
/**
 * Represent a Plugin Hook source file.
 * Plugin hooks are mandatory for Android plugins but are optional for
 * iOS plugins (only needed if the plugin is configurable)
 */
export interface PluginHook {
    /**
     * Name of the plugin hook (name of the source file containing the
     * hook, without extension)
     */
    name: string;
    /**
     * Indicates if this hook is configurable.
     * It tells whether the plugin requires some configuration to be provided
     * upon instantiation by the client code.
     */
    configurable: boolean;
}
/**
 * Android plugin configuration
 */
export interface AndroidPluginConfig extends CommonPluginConfig {
    /**
     * Name of the Android module containing the plugin.
     */
    moduleName: string;
    /**
     * Dependencies (maven artifacts) required by this plugin.
     * Will be added as compile statements in Container build.gradle
     */
    dependencies?: string[];
    /**
     * Repositories to retrieves dependencies from.
     * Will be added to the repositories of the Container build.gradle
     */
    repositories?: string[];
    /**
     * Permissions needed by this plugin.
     * Will be added to the Container Android Manifest file.
     */
    permissions?: string[];
    /**
     * An array of one or more Android hardware or software features.
     * Will be added to the Container Android Manifest file.
     */
    features?: string[];
}
/**
 * iOS plugin configuration
 */
export interface IosPluginConfig extends CommonPluginConfig {
    /**
     * Directives specific to Container pbxproj patching
     */
    pbxproj: PbxProjDirectives;
    /**
     * Set specific build settings in the plugin pbxproj
     */
    setBuildSettings?: IosPluginSetBuildSettingsDirective[];
    /**
     * Array of public headers to add to the container
     */
    containerPublicHeader?: string[];
    /**
     * Path to a Podfile to use for the Container, relative to
     * the directory containing the plugin config.json file
     * Used for RN >= 0.61.0 only
     * Can only be set for 'react-native' plugin configuration
     */
    podfile?: string;
    /**
     * Path to a podspec file to use for the plugin, relative to
     * the directory containing the plugin config.json file
     * Used for RN >= 0.61.0 only
     * Can be used in case a native module doesn't have yet an
     * available podspec file or if the podspec file of the native
     * module needs to be different than the one shipped within
     * the native module
     */
    podspec?: string;
    /**
     * Array of extra pod statements that will be injected in
     * Container Podfile
     */
    extraPods?: string[];
    /**
     * Array of extra pod specs sources that will be injected in
     * Container Podfile
     */
    extraPodspecsSources?: string[];
    /**
     * Array of extra pod statements that will be injected in
     * Container Podfile
     */
    extraPodfileStatements?: string[];
    /**
     * Indicates whether this plugin requires manual linking
     * If that's the case, all plugin directives will be processed
     * independently of the react native version used
     * Otherwise, only `podfile`, `podspec` and `extraPods` directives
     * will be processed
     */
    requiresManualLinking?: boolean;
    /**
     * Indicates whether to ignore the podspec
     * This can be used in case of manually linking a plugin with
     * requiresManualLinking, if the plugin has a podspec
     * Otherwise the plugin will both be manually linked by ern
     * but also auto linked by RN63 due to the precense of the podsepc
     */
    ignorePodSpec?: boolean;
}
/**
 * Set of directives to patch a Container pbxproj
 */
export interface PbxProjDirectives {
    /**
     * Add header file(s) to pbxproj
     */
    addHeader?: IosPluginAddHeaderDirective[];
    /**
     * Add file(s) to the pbxproj
     */
    addFile?: IosPluginAddFileDirective[];
    /**
     * Add Framework(s) to the pbxproj
     */
    addFramework?: string[];
    /**
     * Add Framework(s) search path(s) to the pbxproj
     */
    addFrameworkSearchPath?: string[];
    /**
     * Add Header(s) Search path(s) to the pbxproj
     */
    addHeaderSearchPath?: string[];
    /**
     * Add source file(s) to the pbxproj
     */
    addSource?: IosPluginAddSourceDirective[];
    /**
     * Add static library(ies) to the pbxproj
     */
    addStaticLibrary?: string[];
    /**
     * Add Framework reference(s) to the pbxbproj
     */
    addFrameworkReference?: string[];
    /**
     * Add project(s) to the pbxproj
     */
    addProject?: IosPluginAddProjectDirective[];
    /**
     * Add embedded framework(s) to the pbxproj
     */
    addEmbeddedFramework?: string[];
}
/**
 * Add a file to the Container pbxproj
 */
export interface IosPluginAddFileDirective {
    /**
     * Relative path (from plugin root) to the file to add
     */
    path: string;
    /**
     * Target iOS project group to add the file to
     */
    group: string;
}
/**
 * Add an header to the Container pbxproj
 */
export interface IosPluginAddHeaderDirective {
    /**
     * Source directory and filename widlcard to add
     * ex: IOS/*.swift
     *
     */
    from?: string;
    /**
     * Relative path (from plugin root) to the source file(s) to add
     * If `from` is specified, this is the path of the target directory
     * If `from` is not specified, this is the path of the source file
     */
    path: string;
    /**
     * Target iOS project group to add the header to
     */
    group: string;
    /**
     * Indicates whether the header should be flagged as public or not
     */
    public?: boolean;
}
/**
 * Add a source file the Container pbxproj
 */
export interface IosPluginAddSourceDirective {
    /**
     * Source directory and filename widlcard to add
     * ex: IOS/*.swift
     *
     */
    from?: string;
    /**
     * Target iOS project group to add the source file to
     */
    group: string;
    /**
     * Relative path (from plugin root) to the source file(s) to add
     * If `from` is specified, this is the path of the target directory
     * If `from` is not specified, this is the path of the source file
     */
    path: string;
}
/**
 * Copy a file to the Container
 */
export interface PluginCopyDirective {
    /**
     * Relative path (from plugin root) to the file to copy
     */
    source: string;
    /**
     * Destination path to copy the file to
     */
    dest: string;
}
/**
 * Apply a git patch
 */
export interface PluginApplyPatchDirective {
    /**
     * Relative path (from plugin root) to the patch file to apply
     */
    patch: string;
    /**
     * Relative path (from container out directory) from which to run git apply
     * Mutually exclusive with inNodeModules
     */
    root: string;
    /**
     * If true, root will be set to root location of the plugin in node nodules
     * Mutually exclusive with root
     */
    inNodeModules?: boolean;
}
/**
 * Replace a string in a Container file
 */
export interface PluginReplaceInFileDirective {
    /**
     * Relative path (from plugin root) of the file to
     * apply a string replacement on
     */
    path: string;
    /**
     * The string to replace (can be a RegExp)
     */
    string: string;
    /**
     * The replacement string
     */
    replaceWith: string;
}
/**
 * Add an external project to the Container pbxproj
 */
export interface IosPluginAddProjectDirective {
    /**
     * Indicates whether the project should be added to the
     * Target Dependencies instead of Link Binary With Libraries
     * section of the target project Build Phases
     */
    addAsTargetDependency?: boolean;
    /**
     * Relative path (from plugin root) of the xcodeproj to add
     */
    path: string;
    /**
     * Frameworks to add
     */
    frameworks: string[];
    /**
     * Static lib(s) associated to the project
     */
    staticLibs: IosPluginStaticLib[];
    /**
     * Group to add the xcodeproj to
     */
    group: string;
}
export interface IosPluginBuildSettings {
    /**
     * Configuration(s) for which these build settings apply.
     * For example [ 'Debug' , 'Release' ]
     */
    configurations: string[];
    /**
     * Build settings as key values pairs
     */
    settings: {
        [key: string]: string;
    };
}
export interface IosPluginSetBuildSettingsDirective {
    path: string;
    buildSettings: IosPluginBuildSettings[] | IosPluginBuildSettings;
}
/**
 * iOS Static library representation
 */
export interface IosPluginStaticLib {
    /**
     * Name of the static lib, including extension (.a)
     */
    name: string;
    /**
     * Target of the static lib
     */
    target: string;
}
export interface OverrideManifestConfig {
    url: string;
    type: 'partial' | 'full';
    source: 'cauldron' | '.ernrc';
}
export declare class Manifest {
    static getOverrideManifestConfig: () => Promise<OverrideManifestConfig | void>;
    readonly masterManifest: LocalManifest | GitManifest;
    private manifestOverrideType;
    private overrideManifest;
    constructor(masterManifest: LocalManifest | GitManifest);
    initOverrideManifest(): Promise<void>;
    modifyOverrideManifestUrlIfNeeded(url: string): string;
    hasManifestId(manifestId: string): Promise<boolean>;
    getManifestData({ manifestId, platformVersion, }?: {
        manifestId?: string;
        platformVersion?: string;
    }): Promise<any>;
    getNativeDependencies({ manifestId, platformVersion, }?: {
        manifestId?: string;
        platformVersion?: string;
    }): Promise<PackagePath[]>;
    getJsDependencies({ manifestId, platformVersion, }?: {
        manifestId?: string;
        platformVersion?: string;
    }): Promise<PackagePath[]>;
    getNativeDependency(dependency: PackagePath, { manifestId, platformVersion, }?: {
        manifestId?: string;
        platformVersion?: string;
    }): Promise<PackagePath | void>;
    getJsDependency(dependency: PackagePath, { manifestId, platformVersion, }?: {
        manifestId?: string;
        platformVersion?: string;
    }): Promise<PackagePath | void>;
    getJsAndNativeDependencies({ manifestId, platformVersion, }?: {
        manifestId?: string;
        platformVersion?: string;
    }): Promise<PackagePath[]>;
    getPluginConfigPath(plugin: PackagePath, platformVersion: string): Promise<string | void>;
    isPluginConfigInManifest(plugin: PackagePath, platformVersion: string): Promise<boolean>;
    getPluginConfigFromManifest(plugin: PackagePath, platformVersion: string, projectName: string, platform: NativePlatform): Promise<PluginConfig<'android' | 'ios'> | undefined>;
    getDefaultNpmPluginOrigin(plugin: PackagePath): NpmPluginOrigin;
    getPluginConfig(plugin: PackagePath, platform: 'android', projectName?: string, platformVersion?: string): Promise<PluginConfig<'android'> | undefined>;
    getPluginConfig(plugin: PackagePath, platform: 'ios', projectName?: string, platformVersion?: string): Promise<PluginConfig<'ios'> | undefined>;
    getApiPluginDefaultConfig(plugin: PackagePath, projectName: string | undefined, platform: NativePlatform): PluginConfig<'android' | 'ios'>;
    getApiImplPluginDefaultConfig(plugin: PackagePath, projectName: string | undefined, platform: NativePlatform): PluginConfig<'android' | 'ios'>;
}
export declare const manifest: Manifest;
