"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.manifest = exports.Manifest = void 0;
const PackagePath_1 = require("./PackagePath");
const shell_1 = __importDefault(require("./shell"));
const path_1 = __importDefault(require("path"));
const Platform_1 = __importDefault(require("./Platform"));
const GitManifest_1 = __importDefault(require("./GitManifest"));
const LocalManifest_1 = require("./LocalManifest");
const mustache_1 = __importDefault(require("mustache"));
const lodash_1 = __importDefault(require("lodash"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const utils_1 = require("./utils");
const config_1 = __importDefault(require("./config"));
const log_1 = __importDefault(require("./log"));
const pluginConfigFileName = 'config.json';
const npmScopeModuleRe = /@(.*)\/(.*)/;
const ERN_MANIFEST_MASTER_GIT_REPO = `https://github.com/electrode-io/electrode-native-manifest.git`;
class Manifest {
    constructor(masterManifest) {
        this.masterManifest = masterManifest;
    }
    async initOverrideManifest() {
        if (!this.overrideManifest && Manifest.getOverrideManifestConfig) {
            const overrideManifestConfig = await Manifest.getOverrideManifestConfig();
            if (overrideManifestConfig) {
                const manifestOverrideUrl = this.modifyOverrideManifestUrlIfNeeded(overrideManifestConfig.url);
                this.manifestOverrideType = overrideManifestConfig.type;
                if (await fs_extra_1.default.pathExists(manifestOverrideUrl)) {
                    this.overrideManifest = new LocalManifest_1.LocalManifest(manifestOverrideUrl);
                }
                else {
                    this.overrideManifest = new GitManifest_1.default(Platform_1.default.overrideManifestDirectory, manifestOverrideUrl);
                }
            }
        }
    }
    modifyOverrideManifestUrlIfNeeded(url) {
        // Sample local config :
        // "overrideManifestUrlModifier": {
        //   "searchValue": "github.com",
        //   "replaceValue": "new.github.com"
        // }
        const overrideManifestUrlModifier = config_1.default.get('overrideManifestUrlModifier');
        if (overrideManifestUrlModifier) {
            const obj = JSON.parse(overrideManifestUrlModifier);
            url = url.replace(obj.searchValue, obj.replaceValue);
        }
        return url;
    }
    async hasManifestId(manifestId) {
        await this.initOverrideManifest();
        if (this.overrideManifest && this.manifestOverrideType === 'partial') {
            return ((await this.masterManifest.hasManifestId(manifestId)) ||
                (await this.overrideManifest.hasManifestId(manifestId)));
        }
        else if (this.overrideManifest && this.manifestOverrideType === 'full') {
            return this.overrideManifest.hasManifestId(manifestId);
        }
        else {
            return this.masterManifest.hasManifestId(manifestId);
        }
    }
    async getManifestData({ manifestId = 'default', platformVersion = Platform_1.default.currentVersion, } = {}) {
        await this.initOverrideManifest();
        let manifestData = {};
        if (this.overrideManifest && this.manifestOverrideType === 'partial') {
            // Merge both manifests. If a dependency exists at two different versions in both
            // manifest, the ovveride will take precedence for the version
            const overrideManifestData = await this.overrideManifest.getManifestData({
                manifestId,
                platformVersion,
            });
            const masterManifestData = await this.masterManifest.getManifestData({
                manifestId,
                platformVersion,
            });
            manifestData.targetNativeDependencies = lodash_1.default.unionBy(overrideManifestData
                ? overrideManifestData.targetNativeDependencies
                : [], masterManifestData ? masterManifestData.targetNativeDependencies : [], (d) => PackagePath_1.PackagePath.fromString(d).name);
            manifestData.targetJsDependencies = lodash_1.default.unionBy(overrideManifestData ? overrideManifestData.targetJsDependencies : [], masterManifestData ? masterManifestData.targetJsDependencies : [], (d) => PackagePath_1.PackagePath.fromString(d).name);
        }
        else if (this.overrideManifest && this.manifestOverrideType === 'full') {
            manifestData = await this.overrideManifest.getManifestData({
                manifestId,
                platformVersion,
            });
        }
        else {
            manifestData = await this.masterManifest.getManifestData({
                manifestId,
                platformVersion,
            });
        }
        return manifestData;
    }
    async getNativeDependencies({ manifestId, platformVersion, } = {}) {
        const m = await this.getManifestData({ manifestId, platformVersion });
        return m
            ? lodash_1.default.map(m.targetNativeDependencies, (d) => PackagePath_1.PackagePath.fromString(d))
            : [];
    }
    async getJsDependencies({ manifestId, platformVersion, } = {}) {
        const m = await this.getManifestData({ manifestId, platformVersion });
        return m
            ? lodash_1.default.map(m.targetJsDependencies, (d) => PackagePath_1.PackagePath.fromString(d))
            : [];
    }
    async getNativeDependency(dependency, { manifestId = 'default', platformVersion = Platform_1.default.currentVersion, } = {}) {
        const nativeDependencies = await this.getNativeDependencies({
            manifestId,
            platformVersion,
        });
        return lodash_1.default.find(nativeDependencies, (d) => d.name === dependency.name);
    }
    async getJsDependency(dependency, { manifestId, platformVersion, } = {}) {
        const jsDependencies = await this.getJsDependencies({
            manifestId,
            platformVersion,
        });
        return lodash_1.default.find(jsDependencies, (d) => d.name === dependency.name);
    }
    async getJsAndNativeDependencies({ manifestId, platformVersion, } = {}) {
        const m = await this.getManifestData({ manifestId, platformVersion });
        const manifestDeps = exports.manifest
            ? lodash_1.default.union(m.targetJsDependencies, m.targetNativeDependencies)
            : [];
        return lodash_1.default.map(manifestDeps, (d) => PackagePath_1.PackagePath.fromString(d));
    }
    async getPluginConfigPath(plugin, platformVersion) {
        let pluginConfigPath;
        if (this.overrideManifest && this.manifestOverrideType === 'partial') {
            pluginConfigPath = await this.overrideManifest.getPluginConfigurationPath(plugin, platformVersion);
            if (!pluginConfigPath) {
                pluginConfigPath = await this.masterManifest.getPluginConfigurationPath(plugin, platformVersion);
            }
        }
        else if (this.overrideManifest && this.manifestOverrideType === 'full') {
            pluginConfigPath = await this.overrideManifest.getPluginConfigurationPath(plugin, platformVersion);
        }
        else {
            pluginConfigPath = await this.masterManifest.getPluginConfigurationPath(plugin, platformVersion);
        }
        return pluginConfigPath;
    }
    async isPluginConfigInManifest(plugin, platformVersion) {
        const pluginConfigPath = await this.getPluginConfigPath(plugin, platformVersion);
        return pluginConfigPath !== undefined;
    }
    async getPluginConfigFromManifest(plugin, platformVersion, projectName, platform) {
        var _a, _b;
        const pluginConfigPath = await this.getPluginConfigPath(plugin, platformVersion);
        if (!pluginConfigPath) {
            throw new Error(`There is no configuration for ${plugin.name} plugin in Manifest matching platform version ${platformVersion}`);
        }
        let result;
        let configFile = await fs_extra_1.default.readFile(path_1.default.join(pluginConfigPath, pluginConfigFileName), 'utf-8');
        configFile = mustache_1.default.render(configFile, { projectName });
        result = JSON.parse(configFile);
        // Add default value (convention) for Android subsection for missing fields
        if (platform === 'android' && result.android) {
            const res = result.android;
            res.root = (_a = res.root) !== null && _a !== void 0 ? _a : 'android';
            const matchedFiles = shell_1.default
                .find(pluginConfigPath)
                .filter((file) => {
                return file.match(/\.java$/);
            });
            if (matchedFiles && matchedFiles.length === 1) {
                const pluginHookClass = path_1.default.basename(matchedFiles[0], '.java');
                res.pluginHook = {
                    configurable: false,
                    name: pluginHookClass,
                };
                if (fs_extra_1.default
                    .readFileSync(matchedFiles[0], 'utf-8')
                    .includes('public static class Config')) {
                    res.pluginHook.configurable = true;
                }
            }
            res.path = pluginConfigPath;
            return res;
        }
        else if (platform === 'ios' && result.ios) {
            const res = result.ios;
            res.root = (_b = res.root) !== null && _b !== void 0 ? _b : 'ios';
            const matchedHeaderFiles = shell_1.default
                .find(pluginConfigPath)
                .filter((file) => {
                return file.match(/\.h$/);
            });
            const matchedSourceFiles = shell_1.default
                .find(pluginConfigPath)
                .filter((file) => {
                return file.match(/\.m$/);
            });
            if (matchedHeaderFiles &&
                matchedHeaderFiles.length === 1 &&
                matchedSourceFiles &&
                matchedSourceFiles.length === 1) {
                const pluginHookClass = path_1.default.basename(matchedHeaderFiles[0], '.h');
                res.pluginHook = {
                    configurable: true,
                    name: pluginHookClass,
                };
            }
            res.path = pluginConfigPath;
            return res;
        }
    }
    getDefaultNpmPluginOrigin(plugin) {
        if (npmScopeModuleRe.test(plugin.name)) {
            return {
                name: `${npmScopeModuleRe.exec(plugin.name)[2]}`,
                scope: `${npmScopeModuleRe.exec(plugin.name)[1]}`,
                type: 'npm',
                version: plugin.version,
            };
        }
        else {
            return {
                name: plugin.name,
                type: 'npm',
                version: plugin.version,
            };
        }
    }
    async getPluginConfig(plugin, platform, projectName = 'ElectrodeContainer', platformVersion = Platform_1.default.currentVersion) {
        await this.initOverrideManifest();
        let result;
        if (await (0, utils_1.isDependencyApi)(plugin)) {
            log_1.default.debug(`Using default config for API plugin: ${plugin.name}@${plugin.version}.`);
            result = this.getApiPluginDefaultConfig(plugin, projectName, platform);
        }
        else if (await (0, utils_1.isDependencyApiImpl)(plugin)) {
            log_1.default.debug(`Using default config for APIImpl plugin: ${plugin.name}@${plugin.version}.`);
            result = this.getApiImplPluginDefaultConfig(plugin, projectName, platform);
        }
        else if (await this.isPluginConfigInManifest(plugin, platformVersion)) {
            log_1.default.debug(`Using config from manifest for third party plugin: ${plugin.name}@${plugin.version}.`);
            result = await this.getPluginConfigFromManifest(plugin, platformVersion, projectName, platform);
        }
        else {
            log_1.default.warn(`Unsupported plugin. No configuration found in manifest for ${plugin.name}@${plugin.version}.`);
            return;
        }
        if (result && !result.origin) {
            result.origin = this.getDefaultNpmPluginOrigin(plugin);
        }
        else if (result && !result.origin.version) {
            result.origin.version = plugin.version;
        }
        return result;
    }
    getApiPluginDefaultConfig(plugin, projectName = 'UNKNOWN', platform) {
        // APIs generated with ERN <= 0.49 use upper case 'IOS'
        const iosPath = fs_extra_1.default.pathExistsSync(`${plugin.fullPath}/ios`) ? 'ios' : 'IOS';
        return platform === 'android'
            ? {
                moduleName: 'lib',
                origin: this.getDefaultNpmPluginOrigin(plugin),
                root: 'android',
            }
            : {
                copy: [
                    {
                        dest: `${projectName}/APIs`,
                        source: `${iosPath}/*`,
                    },
                ],
                origin: this.getDefaultNpmPluginOrigin(plugin),
                pbxproj: {
                    addHeader: [
                        {
                            from: `${iosPath}/*.swift`,
                            group: 'APIs',
                            path: 'APIs',
                            public: true,
                        },
                        {
                            from: `${iosPath}/*.h`,
                            group: 'APIs',
                            path: 'APIs',
                            public: true,
                        },
                    ],
                    addSource: [
                        {
                            from: `${iosPath}/*.swift`,
                            group: 'APIs',
                            path: 'APIs',
                        },
                        {
                            from: `${iosPath}/*.m`,
                            group: 'APIs',
                            path: 'APIs',
                        },
                    ],
                },
                requiresManualLinking: true,
                root: 'ios',
            };
    }
    getApiImplPluginDefaultConfig(plugin, projectName = 'UNKNOWN', platform) {
        return platform === 'android'
            ? {
                moduleName: 'lib',
                origin: this.getDefaultNpmPluginOrigin(plugin),
                root: 'android',
            }
            : {
                copy: [
                    {
                        dest: `${projectName}/APIImpls`,
                        source: 'ios/ElectrodeApiImpl/APIImpls/*',
                    },
                ],
                origin: this.getDefaultNpmPluginOrigin(plugin),
                pbxproj: {
                    addSource: [
                        {
                            from: 'ios/ElectrodeApiImpl/APIImpls/*.swift',
                            group: 'APIImpls',
                            path: 'APIImpls',
                        },
                    ],
                },
                requiresManualLinking: true,
                root: 'ios',
            };
    }
}
exports.Manifest = Manifest;
const manifestLocalConfig = config_1.default.get('manifest', {});
const manifestLocalMasterUrl = (_a = manifestLocalConfig === null || manifestLocalConfig === void 0 ? void 0 : manifestLocalConfig.master) === null || _a === void 0 ? void 0 : _a.url;
exports.manifest = manifestLocalMasterUrl
    ? new Manifest(new LocalManifest_1.LocalManifest(manifestLocalMasterUrl))
    : new Manifest(new GitManifest_1.default(Platform_1.default.masterManifestDirectory, ERN_MANIFEST_MASTER_GIT_REPO));
//# sourceMappingURL=Manifest.js.map