"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModuleFactory = void 0;
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const shell_1 = __importDefault(require("./shell"));
const clients_1 = require("./clients");
const PackagePath_1 = require("./PackagePath");
const packageJsonFileUtils_1 = require("./packageJsonFileUtils");
class ModuleFactory {
    constructor(packageCachePath, { packagePrefix, } = {}) {
        this.packageCachePath = packageCachePath;
        this.packagePrefix = packagePrefix;
    }
    async getModuleInstance(p) {
        if (!p.isFilePath && !p.isRegistryPath) {
            throw new Error(`Not a supported package path : ${p.toString()}`);
        }
        const pathToModule = await this.getLocalPathToPackage(p);
        return this.instantiateModule(pathToModule);
    }
    instantiateModule(pathToModule) {
        const Module = require(pathToModule).default;
        return new Module();
    }
    async getLocalPathToPackage(p) {
        return p.isFilePath
            ? this.getPathToLocalPackage(p)
            : await this.getPathToRegistryPackage(p);
    }
    getPathToLocalPackage(p) {
        const pathWithSrc = path_1.default.join(p.basePath, 'src');
        return fs_extra_1.default.pathExistsSync(pathWithSrc) ? pathWithSrc : p.basePath;
    }
    async getPathToRegistryPackage(p) {
        if (!this.doesPackageCacheExist()) {
            await this.createPackageCache();
        }
        const modulePackagePath = this.packagePrefix
            ? this.processPackageRegistryPath(p)
            : p;
        await this.refreshCacheFor(modulePackagePath);
        return path_1.default.join(this.packageCachePath, 'node_modules', modulePackagePath.basePath);
    }
    doesPackageCacheExist() {
        return fs_extra_1.default.pathExistsSync(this.packageCachePath);
    }
    async createPackageCache() {
        shell_1.default.mkdir('-p', this.packageCachePath);
        try {
            shell_1.default.pushd(this.packageCachePath);
            await clients_1.yarn.init();
        }
        finally {
            shell_1.default.popd();
        }
    }
    async refreshCacheFor(p) {
        var _a;
        const packageJson = await (0, packageJsonFileUtils_1.readPackageJson)(this.packageCachePath);
        return ((_a = packageJson.dependencies) === null || _a === void 0 ? void 0 : _a[p.basePath])
            ? this.upgradeCachedPackage(p)
            : this.addPackageToCache(p);
    }
    async addPackageToCache(p) {
        shell_1.default.pushd(this.packageCachePath);
        try {
            await clients_1.yarn.add(p);
        }
        finally {
            shell_1.default.popd();
        }
    }
    async upgradeCachedPackage(p) {
        shell_1.default.pushd(this.packageCachePath);
        try {
            await clients_1.yarn.upgrade(p);
        }
        finally {
            shell_1.default.popd();
        }
    }
    processPackageRegistryPath(p) {
        if (!p.fullPath.startsWith('@') &&
            !p.fullPath.startsWith(this.packagePrefix)) {
            return PackagePath_1.PackagePath.fromString(`${this.packagePrefix}${p.fullPath}`);
        }
        return p;
    }
}
exports.ModuleFactory = ModuleFactory;
//# sourceMappingURL=ModuleFactory.js.map