"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PackagePath = void 0;
const untildify_1 = __importDefault(require("untildify"));
const packageJsonFileUtils_1 = require("./packageJsonFileUtils");
const FILE_PATH_WITH_PREFIX_RE = /^file:(.+)/;
const FILE_PATH_WITHOUT_PREFIX_RE = /^(\/.+)/;
const FILE_PATH_TIDLE_RE = /^(~\/.+)/;
const FILE_PATH_WITHOUT_PREFIX_WINDOWS_RE = /^([a-zA-Z]:\\[\\\S|*\S]?.*)$/;
const FILE_PATH_RE = new RegExp(`${FILE_PATH_WITH_PREFIX_RE.source}|${FILE_PATH_WITHOUT_PREFIX_RE.source}|${FILE_PATH_WITHOUT_PREFIX_WINDOWS_RE.source}|${FILE_PATH_TIDLE_RE.source}`);
const GIT_SSH_PATH_RE = new RegExp(/^git\+ssh:\/\/.+$/);
const GIT_SSH_PATH_VERSION_RE = new RegExp(/^(git\+ssh:\/\/.+)#(.+)$/);
const GIT_HTTPS_PATH_RE = new RegExp(/^https:\/\/.+$/);
const GIT_HTTPS_PATH_VERSION_RE = new RegExp(/^(https:\/\/.+)#(.+)$/);
const GITHUB_SSH_PATH_RE = new RegExp(/^git@[^:]+:[^\/]+\/.+\.git$/);
const GITHUB_SSH_PATH_VERSION_RE = new RegExp(/^git@[^:]+:[^\/]+\/.+\.git#.+$/);
const GIT_PATH_RE = new RegExp(`${GIT_SSH_PATH_RE.source}|${GIT_SSH_PATH_VERSION_RE.source}|${GIT_HTTPS_PATH_RE.source}|${GIT_HTTPS_PATH_VERSION_RE.source}|${GITHUB_SSH_PATH_VERSION_RE.source}|${GITHUB_SSH_PATH_RE.source}`);
const REGISTRY_PATH_VERSION_RE = new RegExp(/^(.+)@(.+)$/);
class PackagePath {
    /**
     * Creates a PackagePath from a string
     * @param path Full path to the package (registry|git|file)
     */
    static fromString(path) {
        return new PackagePath(path);
    }
    /**
     * Creates a PackagePath
     * @param path Full path to the package (registry|git|file)
     */
    constructor(path) {
        // Transform GitHub SSH urls
        if (GITHUB_SSH_PATH_VERSION_RE.test(path) ||
            GITHUB_SSH_PATH_RE.test(path)) {
            path = `git+ssh://${path.replace(':', '/')}`;
        }
        if (GIT_SSH_PATH_VERSION_RE.test(path)) {
            this.basePath = GIT_SSH_PATH_VERSION_RE.exec(path)[1];
            this.version = GIT_SSH_PATH_VERSION_RE.exec(path)[2];
        }
        else if (GIT_HTTPS_PATH_VERSION_RE.test(path)) {
            this.basePath = GIT_HTTPS_PATH_VERSION_RE.exec(path)[1];
            this.version = GIT_HTTPS_PATH_VERSION_RE.exec(path)[2];
        }
        else if (GIT_SSH_PATH_RE.test(path) || GIT_HTTPS_PATH_RE.test(path)) {
            this.basePath = path;
        }
        else if (FILE_PATH_WITH_PREFIX_RE.test(path)) {
            this.basePath = FILE_PATH_WITH_PREFIX_RE.exec(path)[1];
        }
        else if (FILE_PATH_WITHOUT_PREFIX_RE.test(path)) {
            this.basePath = FILE_PATH_WITHOUT_PREFIX_RE.exec(path)[1];
        }
        else if (FILE_PATH_WITHOUT_PREFIX_WINDOWS_RE.test(path)) {
            this.basePath = FILE_PATH_WITHOUT_PREFIX_WINDOWS_RE.exec(path)[1];
        }
        else if (FILE_PATH_TIDLE_RE.test(path)) {
            this.basePath = (0, untildify_1.default)(FILE_PATH_TIDLE_RE.exec(path)[1]);
        }
        else if (REGISTRY_PATH_VERSION_RE.test(path)) {
            this.basePath = REGISTRY_PATH_VERSION_RE.exec(path)[1];
            this.name = REGISTRY_PATH_VERSION_RE.exec(path)[1];
            this.version = REGISTRY_PATH_VERSION_RE.exec(path)[2];
        }
        else {
            this.basePath = path;
            this.name = path;
        }
        this.fullPath = path;
        if (FILE_PATH_RE.test(this.fullPath)) {
            const pJson = (0, packageJsonFileUtils_1.readPackageJsonSync)(this.basePath);
            this.version = pJson.version;
            this.name = pJson.name;
        }
    }
    get isGitPath() {
        return GIT_PATH_RE.test(this.fullPath);
    }
    get isFilePath() {
        return FILE_PATH_RE.test(this.fullPath);
    }
    get isRegistryPath() {
        return !this.isGitPath && !this.isFilePath;
    }
    same(other, { ignoreVersion, } = {}) {
        return (this.basePath === other.basePath &&
            (ignoreVersion ? true : this.version === other.version));
    }
    toString() {
        return this.fullPath;
    }
}
exports.PackagePath = PackagePath;
//# sourceMappingURL=PackagePath.js.map