"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = __importDefault(require("./config"));
const shell_1 = __importDefault(require("./shell"));
const log_1 = __importDefault(require("./log"));
const child_process_1 = require("child_process");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const os_1 = __importDefault(require("os"));
const semver_1 = __importDefault(require("semver"));
const ROOT_DIRECTORY = () => process.env.ERN_HOME || path_1.default.join(os_1.default.homedir(), '.ern');
// Name of ern local client NPM package
const ERN_LOCAL_CLI_PACKAGE = 'ern-local-cli';
class Platform {
    static get rootDirectory() {
        return ROOT_DIRECTORY();
    }
    static get packagesCacheDirectory() {
        return path_1.default.join(this.rootDirectory, 'packages-cache');
    }
    static get cauldronDirectory() {
        return path_1.default.join(this.rootDirectory, 'cauldron');
    }
    static get localCauldronsDirectory() {
        return path_1.default.join(this.rootDirectory, 'local-cauldrons');
    }
    static get masterManifestDirectory() {
        return path_1.default.join(this.rootDirectory, 'ern-master-manifest');
    }
    static get overrideManifestDirectory() {
        return path_1.default.join(this.rootDirectory, 'ern-override-manifest');
    }
    static get versionCacheDirectory() {
        return path_1.default.join(this.rootDirectory, 'versions');
    }
    static get containerPublishersCacheDirectory() {
        return path_1.default.join(this.rootDirectory, 'container-publishers-cache');
    }
    static get containerTransformersCacheDirectory() {
        return path_1.default.join(this.rootDirectory, 'container-transformers-cache');
    }
    static get containerGenDirectory() {
        return path_1.default.join(this.rootDirectory, 'containergen');
    }
    static getContainerGenOutDirectory(platform) {
        return path_1.default.join(this.containerGenDirectory, 'out', platform);
    }
    static get latestVersion() {
        return this.versions.slice(-1)[0];
    }
    static get currentPlatformVersionPath() {
        return this.getPlatformVersionPath(this.currentVersion);
    }
    static get currentVersion() {
        return config_1.default.get('platformVersion', '1000.0.0');
    }
    static isPlatformVersionAvailable(version) {
        return this.versions.includes('' + version);
    }
    static getRootPlatformVersionPath(version) {
        return path_1.default.join(this.versionCacheDirectory, version);
    }
    static isPlatformVersionInstalled(version) {
        return fs_extra_1.default.pathExistsSync(this.getRootPlatformVersionPath(version));
    }
    static getPlatformVersionPath(version) {
        return version === '1000.0.0'
            ? path_1.default.join(this.versionCacheDirectory, version)
            : path_1.default.join(this.versionCacheDirectory, version, 'node_modules');
    }
    // Return an array of versions (ex: [1,2,3,4,5])
    // representing all the available versions of the platform local cli
    static get versions() {
        return JSON.parse((0, child_process_1.execSync)(`npm info ${ERN_LOCAL_CLI_PACKAGE} versions --json`).toString()).sort(semver_1.default.compare);
    }
    // Install a given platform version
    static installPlatform(version) {
        version = this.normalizeVersion(version);
        if (this.isPlatformVersionInstalled(version)) {
            return log_1.default.warn(`Version ${version} of ern platform is already installed`);
        }
        if (!this.isPlatformVersionAvailable(version)) {
            throw new Error(`Version ${version} of ern platform is not available`);
        }
        const pathToVersion = this.getRootPlatformVersionPath(version);
        try {
            shell_1.default.mkdir(pathToVersion);
            shell_1.default.pushd(pathToVersion);
            if (this.isYarnInstalled()) {
                // Favor yarn if it is installed as it will greatly speed up install
                (0, child_process_1.execSync)(`yarn init --yes`, { cwd: pathToVersion });
                (0, child_process_1.execSync)(`yarn add ${ERN_LOCAL_CLI_PACKAGE}@${version} --exact --ignore-engines`, {
                    cwd: pathToVersion,
                });
            }
            else {
                (0, child_process_1.execSync)(`npm init --yes`, { cwd: pathToVersion });
                (0, child_process_1.execSync)(`npm install ${ERN_LOCAL_CLI_PACKAGE}@${version} --exact`, {
                    cwd: pathToVersion,
                });
            }
        }
        catch (e) {
            log_1.default.error('Something went wrong during installation. Performing clean up.');
            shell_1.default.rm('-rf', pathToVersion);
            throw e;
        }
        finally {
            shell_1.default.popd();
        }
    }
    // Uninstall a given platform version
    static uninstallPlatform(version) {
        if (!this.isPlatformVersionInstalled(version)) {
            return log_1.default.warn(`Version ${version} of ern platform is not installed`);
        }
        if (this.currentVersion === version) {
            return log_1.default.error(`Version ${version} is currently activated. Cannot uninstall`);
        }
        shell_1.default.rm('-rf', this.getRootPlatformVersionPath(version));
    }
    // Switch to / activate a given version
    // If the version is not installed yet, it will install it beforehand, then
    // it will just update the config file with new activated version number
    static switchToVersion(version) {
        version = this.normalizeVersion(version);
        if (version === this.currentVersion) {
            return log_1.default.info(`Already using ern v${version}`);
        }
        if (!this.isPlatformVersionInstalled(version)) {
            if (!this.isPlatformVersionAvailable(version)) {
                throw new Error(`Version ${version} of ern platform is not available`);
            }
            log_1.default.info(`ern v${version} is not installed yet. Installing now.`);
            this.installPlatform(version);
        }
        config_1.default.set('platformVersion', version);
        log_1.default.info(`ern v${version} is now activated.`);
    }
    static normalizeVersion(version) {
        let result;
        if (version === 'latest') {
            result = Platform.versions.pop();
        }
        else {
            result = semver_1.default.valid(version);
            if (!result) {
                throw new Error(`Cannot normalize invalid version : ${version}`);
            }
        }
        return result;
    }
    static isYarnInstalled() {
        try {
            if (process.platform === 'win32') {
                (0, child_process_1.execSync)('yarn --version > NUL 2>&1');
            }
            else {
                (0, child_process_1.execSync)('yarn --version 1>/dev/null 2>/dev/null');
            }
            return true;
        }
        catch (e) {
            return false;
        }
    }
    static isCocoaPodsInstalled() {
        try {
            (0, child_process_1.execSync)('pod --version 1>/dev/null 2>/dev/null');
            return true;
        }
        catch (e) {
            return false;
        }
    }
}
exports.default = Platform;
//# sourceMappingURL=Platform.js.map