"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const shell_1 = __importDefault(require("./shell"));
const createTmpDir_1 = __importDefault(require("./createTmpDir"));
const childProcess_1 = require("./childProcess");
const child_process_1 = require("child_process");
const node_fetch_1 = __importDefault(require("node-fetch"));
const log_1 = __importDefault(require("./log"));
const kax_1 = __importDefault(require("./kax"));
const util_1 = __importDefault(require("util"));
const semver_1 = __importDefault(require("semver"));
const os_1 = __importDefault(require("os"));
const ex = util_1.default.promisify(child_process_1.exec);
const sp = util_1.default.promisify(child_process_1.spawn);
class ReactNativeCli {
    constructor(binaryPath = 'react-native') {
        this.binaryPath = binaryPath;
    }
    async init(projectName, rnVersion, { skipInstall, template, } = {}) {
        const dir = path_1.default.join(process.cwd(), projectName);
        if (await fs_extra_1.default.pathExists(dir)) {
            throw new Error(`Path already exists will not override ${dir}`);
        }
        const options = [];
        options.push(`--version ${rnVersion}`);
        if (skipInstall) {
            options.push('--skip-install');
        }
        if (template) {
            options.push(`--template ${template}`);
        }
        const initCmd = `init ${projectName} ${options.join(' ')}`;
        if (semver_1.default.gte(rnVersion, '0.60.0')) {
            return (0, childProcess_1.execp)(`npx --ignore-existing react-native@${rnVersion} ${initCmd}`);
        }
        else {
            return (0, childProcess_1.execp)(`${this.binaryPath} ${initCmd}`);
        }
    }
    async bundle({ entryFile, dev, bundleOutput, assetsDest, platform, workingDir, sourceMapOutput, resetCache, }) {
        const bundleCommand = `${this.binaryPath} bundle \
${entryFile ? `--entry-file=${entryFile}` : ''} \
${dev ? '--dev=true' : '--dev=false'} \
${platform ? `--platform=${platform}` : ''} \
${bundleOutput ? `--bundle-output=${bundleOutput}` : ''} \
${assetsDest ? `--assets-dest=${assetsDest}` : ''} \
${sourceMapOutput ? `--sourcemap-output=${sourceMapOutput}` : ''} \
${resetCache ? '--reset-cache' : ''}`;
        await (0, childProcess_1.execp)(bundleCommand, { cwd: workingDir });
        if (!(await fs_extra_1.default.pathExists(bundleOutput))) {
            // Under some circumstances, Metro bundler process might fail
            // with some logs, but exit the process with a non error status code.
            // This guard is to make sure that the bundle was generated,
            // independently of the exit status code returned by Metro process.
            throw new Error('Metro failed to generate the JS bundle. Check Metro logs for more details.');
        }
        return {
            assetsPath: assetsDest,
            bundlePath: bundleOutput,
            dev,
            platform,
            sourceMapPath: sourceMapOutput,
        };
    }
    startPackager({ cwd = process.cwd(), host = 'localhost', port = '8081', resetCache = true, } = {}) {
        const args = [];
        if (host) {
            args.push('--host', host);
        }
        if (port) {
            args.push('--port', port);
        }
        if (resetCache) {
            args.push(`--reset-cache`);
        }
        (0, child_process_1.spawn)(path_1.default.join(cwd, `node_modules/.bin/react-native${os_1.default.platform() === 'win32' ? '.cmd' : ''}`), ['start', ...args], {
            cwd,
            stdio: 'inherit',
        });
    }
    async startPackagerInNewWindow({ cwd = process.cwd(), host = 'localhost', port = '8081', resetCache = true, provideModuleNodeModules, } = {}) {
        const args = [];
        if (host) {
            args.push(`--host ${host}`);
        }
        if (port) {
            args.push(`--port ${port}`);
        }
        if (resetCache) {
            args.push(`--reset-cache`);
        }
        if (provideModuleNodeModules) {
            args.push(`--providesModuleNodeModules ${provideModuleNodeModules.join(',')}`);
        }
        const isPackagerRunning = await this.isPackagerRunning(host, port);
        if (!isPackagerRunning) {
            await kax_1.default
                .task(`Starting React Native Packager [http://${host}:${port}]`)
                .run(Promise.resolve());
            if (process.platform === 'darwin') {
                return this.darwinStartPackagerInNewWindow({ cwd, args });
            }
            else if (/^win/.test(process.platform)) {
                return this.windowsStartPackagerInNewWindow({ cwd, args });
            }
            else {
                return this.linuxStartPackageInNewWindow({ cwd, args });
            }
        }
        else {
            log_1.default.warn('A React Native Packager is already running in a different process');
        }
    }
    async darwinStartPackagerInNewWindow({ cwd = process.cwd(), args = [], }) {
        const scriptPath = await this.createStartPackagerScript({
            args,
            cwd,
            scriptFileName: 'packager.sh',
        });
        (0, childProcess_1.spawnp)('open', ['-a', 'Terminal', scriptPath]);
    }
    async linuxStartPackageInNewWindow({ cwd = process.cwd(), args = [], }) {
        const scriptPath = await this.createStartPackagerScript({
            args,
            cwd,
            scriptFileName: 'packager.sh',
        });
        (0, childProcess_1.spawnp)('gnome-terminal', ['--command', scriptPath]);
    }
    async windowsStartPackagerInNewWindow({ cwd = process.cwd(), args = [], }) {
        const scriptPath = await this.createStartPackagerScript({
            args,
            cwd,
            scriptFileName: 'packager.bat',
        });
        (0, childProcess_1.spawnp)('cmd.exe', ['/C', scriptPath], { detached: true });
    }
    async createStartPackagerScript({ cwd, args, scriptFileName, }) {
        const tmpDir = (0, createTmpDir_1.default)();
        const tmpScriptPath = path_1.default.join(tmpDir, scriptFileName);
        await fs_extra_1.default.writeFile(tmpScriptPath, `
cd ${cwd}
echo "Running ${this.binaryPath} start ${args.join(' ')}"
${this.binaryPath} start ${args.join(' ')}
`);
        shell_1.default.chmod('+x', tmpScriptPath);
        return tmpScriptPath;
    }
    async isPackagerRunning(host, port) {
        return (0, node_fetch_1.default)(`http://${host}:${port}/status`).then((res) => res.text().then((body) => body === 'packager-status:running'), () => false);
    }
}
exports.default = ReactNativeCli;
//# sourceMappingURL=ReactNativeCli.js.map