"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.YarnCli = void 0;
const createTmpDir_1 = __importDefault(require("./createTmpDir"));
const shell_1 = __importDefault(require("./shell"));
const path_1 = __importDefault(require("path"));
const PackagePath_1 = require("./PackagePath");
const childProcess_1 = require("./childProcess");
const log_1 = __importDefault(require("./log"));
const cross_spawn_1 = require("cross-spawn");
const packageJsonFileUtils_1 = require("./packageJsonFileUtils");
class YarnCli {
    constructor(binaryPath = 'yarn') {
        this.binaryPath = binaryPath;
    }
    async add(dependencyPath, { dev, peer, } = {}) {
        // Special handling with yarn add when the dependency is a local file path
        // In that case, for some reason it copies the node_modules directory of this path, which
        // is not a wanted behavior, especially for react-native bundling as it will create
        // haste module naming collisions
        // As a temporaray work-around, we copy the whole package directory to a temporary one
        // and remove node_modules from there and use this new path instead when yarn adding
        // Issue has been opened here https://github.com/yarnpkg/yarn/issues/1334
        // (still open as of this comment writing)
        // [Note: We tried another lighter workaround being to just remove the node_modules
        // directory contained within this package after yarn add is executed. Howver subsequent
        // yarn add of a different dependency just reintroduce the error on previous package
        // this is really weird]
        if (dependencyPath.isFilePath) {
            const tmpDirPath = (0, createTmpDir_1.default)();
            shell_1.default.cp('-R', path_1.default.join(dependencyPath.basePath, '{.*,*}'), tmpDirPath);
            shell_1.default.rm('-rf', path_1.default.join(tmpDirPath, 'node_modules'));
            dependencyPath = PackagePath_1.PackagePath.fromString(`file:${tmpDirPath}`);
        }
        const cmd = `add ${dependencyPath.toString()} --ignore-engines --non-interactive --exact ${dev ? '--dev' : ''} ${peer ? '--peer' : ''}`;
        return this.runYarnCommand(cmd);
    }
    async install() {
        const cmd = `install --ignore-engines`;
        return this.runYarnCommand(cmd);
    }
    async upgrade(dependencyPath) {
        const cmd = `upgrade ${dependencyPath.toString()} --ignore-engines --exact`;
        return this.runYarnCommand(cmd);
    }
    async init() {
        const cmd = `init --yes`;
        return this.runYarnCommand(cmd);
    }
    async info(dependencyPath, { field, } = {}) {
        log_1.default.trace(`[YarnCli] info(${dependencyPath}, {field: ${field}})`);
        if (dependencyPath.isFilePath) {
            const pJson = await (0, packageJsonFileUtils_1.readPackageJson)(dependencyPath.basePath);
            return field ? pJson[field] : pJson;
        }
        else {
            const args = [
                'info',
                dependencyPath.toString(),
                ...(field ? [field] : []),
                '--json',
            ];
            return new Promise((resolve, reject) => {
                const cp = (0, cross_spawn_1.spawn)(this.binaryPath, args);
                cp.stdout.on('data', (data) => {
                    log_1.default.trace(data);
                    const jsonLine = JSON.parse(data.toString());
                    if (jsonLine.type === 'inspect') {
                        resolve(jsonLine.data);
                    }
                });
                cp.stderr.on('data', (data) => {
                    log_1.default.trace(data);
                    const jsonLine = JSON.parse(data.toString());
                    // 'warning' and 'error' packet types are sent to stderr
                    // we want to fail only on 'error'
                    if (jsonLine.type === 'error') {
                        if (jsonLine.data.includes('Received invalid response from npm.')) {
                            reject(new Error('Package not found'));
                        }
                        else {
                            reject(jsonLine.data);
                        }
                    }
                });
            });
        }
    }
    async runYarnCommand(command) {
        const cmd = `${this.binaryPath} ${command}`;
        log_1.default.debug(`[runYarnCommand] Running ${cmd}`);
        return (0, childProcess_1.execp)(cmd);
    }
}
exports.YarnCli = YarnCli;
//# sourceMappingURL=YarnCli.js.map