"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveNativeDependenciesVersionsEx = exports.resolveNativeDependenciesVersions = exports.resolvePackageVersionsGivenMismatchLevel = exports.retainHighestVersions = exports.containsVersionMismatch = void 0;
const semver_1 = __importDefault(require("semver"));
const lodash_1 = __importDefault(require("lodash"));
function containsVersionMismatch(versions, mismatchLevel) {
    const semverVersions = versions.map((v) => semver_1.default.parse(v));
    const hasMajorDiff = lodash_1.default.uniqBy(semverVersions, 'major').length > 1;
    const hasMinorDiff = lodash_1.default.uniqBy(semverVersions, 'minor').length > 1;
    const hasPatchDiff = lodash_1.default.uniqBy(semverVersions, 'patch').length > 1;
    const hasPreReleaseDiff = lodash_1.default.uniqWith(semverVersions.map((v) => v.prerelease), lodash_1.default.isEqual).length > 1;
    return mismatchLevel === 'patch'
        ? hasMajorDiff || hasMinorDiff || hasPatchDiff || hasPreReleaseDiff
        : mismatchLevel === 'minor'
            ? hasMajorDiff || hasMinorDiff || hasPreReleaseDiff
            : hasMajorDiff || hasPreReleaseDiff;
}
exports.containsVersionMismatch = containsVersionMismatch;
function retainHighestVersions(dependenciesA, dependenciesB) {
    const result = [];
    const groups = lodash_1.default.groupBy([...dependenciesA, ...dependenciesB], 'name');
    let dependencyGroup;
    for (dependencyGroup of Object.values(groups)) {
        let highestVersionDependency = dependencyGroup[0];
        if (dependencyGroup.length > 1 &&
            semver_1.default.gt(dependencyGroup[1].version, dependencyGroup[0].version)) {
            highestVersionDependency = dependencyGroup[1];
        }
        result.push(highestVersionDependency);
    }
    return result;
}
exports.retainHighestVersions = retainHighestVersions;
function resolvePackageVersionsGivenMismatchLevel(plugins, mismatchLevel) {
    const result = {
        pluginsWithMismatchingVersions: [],
        resolved: [],
    };
    const pluginsByName = lodash_1.default.groupBy(lodash_1.default.unionBy(plugins, (p) => p.toString()), 'name');
    for (const name of Object.keys(pluginsByName)) {
        const entry = pluginsByName[name];
        const pluginVersions = lodash_1.default.uniq(lodash_1.default.map(entry, 'version'));
        if (pluginVersions.length > 1) {
            // If there are multiple versions of the dependency
            if ((name !== 'react-native-electrode-bridge' &&
                containsVersionMismatch(pluginVersions, mismatchLevel)) ||
                (name === 'react-native-electrode-bridge' &&
                    containsVersionMismatch(pluginVersions, 'major'))) {
                // If at least one of the versions major digit differs, deem incompatibility
                result.pluginsWithMismatchingVersions.push(name);
            }
            else {
                // No mismatchLevel version differences, just return the highest version
                result.resolved.push(lodash_1.default.find(entry, (c) => c.name === name &&
                    c.version === semver_1.default.maxSatisfying(pluginVersions, '*')));
            }
        }
        else {
            // Only one version is used across all MiniApps, just use this version
            result.resolved.push(entry[0]);
        }
    }
    return result;
}
exports.resolvePackageVersionsGivenMismatchLevel = resolvePackageVersionsGivenMismatchLevel;
function resolveNativeDependenciesVersions(nativeDependenciesArr) {
    const aggregateNativeDependencies = {
        all: [],
        apis: [],
        nativeApisImpl: [],
        thirdPartyInManifest: [],
        thirdPartyNotInManifest: [],
    };
    // Build a map of all the native dependencies of each of the MiniApps
    for (const nativeDependencies of nativeDependenciesArr) {
        aggregateNativeDependencies.apis.push(...nativeDependencies.apis);
        aggregateNativeDependencies.nativeApisImpl.push(...nativeDependencies.nativeApisImpl);
        aggregateNativeDependencies.thirdPartyInManifest.push(...nativeDependencies.thirdPartyInManifest);
        aggregateNativeDependencies.thirdPartyNotInManifest.push(...nativeDependencies.thirdPartyNotInManifest);
    }
    return resolveNativeDependenciesVersionsEx(aggregateNativeDependencies);
}
exports.resolveNativeDependenciesVersions = resolveNativeDependenciesVersions;
function resolveNativeDependenciesVersionsEx(dependencies) {
    // Resolve native dependencies versions of APIs / APIs impls
    let apisAndApiImplsNativeDeps = [];
    if (dependencies.apis.length > 0) {
        apisAndApiImplsNativeDeps.push(...lodash_1.default.flatten(dependencies.apis));
    }
    if (dependencies.nativeApisImpl.length > 0) {
        apisAndApiImplsNativeDeps.push(...lodash_1.default.flatten(dependencies.nativeApisImpl));
    }
    apisAndApiImplsNativeDeps = lodash_1.default.flatten(apisAndApiImplsNativeDeps);
    const apiAndApiImplsResolvedVersions = resolvePackageVersionsGivenMismatchLevel(apisAndApiImplsNativeDeps, 'major');
    // Resolve native dependencies versions third party native modules
    let thirdPartyNativeModules = [];
    if (dependencies.thirdPartyInManifest.length > 0) {
        thirdPartyNativeModules.push(...lodash_1.default.flatten(dependencies.thirdPartyInManifest));
    }
    if (dependencies.thirdPartyNotInManifest.length > 0) {
        thirdPartyNativeModules.push(...lodash_1.default.flatten(dependencies.thirdPartyNotInManifest));
    }
    thirdPartyNativeModules = lodash_1.default.flatten(thirdPartyNativeModules);
    const thirdPartyNativeModulesResolvedVersions = resolvePackageVersionsGivenMismatchLevel(thirdPartyNativeModules, 'patch');
    return {
        pluginsWithMismatchingVersions: [
            ...apiAndApiImplsResolvedVersions.pluginsWithMismatchingVersions,
            ...thirdPartyNativeModulesResolvedVersions.pluginsWithMismatchingVersions,
        ],
        resolved: [
            ...apiAndApiImplsResolvedVersions.resolved,
            ...thirdPartyNativeModulesResolvedVersions.resolved,
        ],
    };
}
exports.resolveNativeDependenciesVersionsEx = resolveNativeDependenciesVersionsEx;
//# sourceMappingURL=resolveNativeDependenciesVersions.js.map