"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCommitShaOfGitPackage = exports.getCommitShaOfGitBranchOrTag = exports.isGitTag = exports.isGitBranch = exports.coerceToPackagePath = exports.coerceToPackagePathArray = exports.coerceToAnyAppDescriptor = exports.coerceToAppVersionDescriptor = exports.coerceToAppPlatformDescriptor = exports.coerceToAppNameDescriptor = exports.coerceToAppVersionDescriptorArray = exports.coerceToAnyAppDescriptorArray = exports.coerceToAppPlatformDescriptorArray = exports.coerceToAppNameDescriptorArray = exports.logErrorAndExitProcess = exports.isValidElectrodeNativeModuleName = exports.reactNativeManifestVersion = exports.isDependencyPathNativeApiImpl = exports.isDependencyPathJsApiImpl = exports.isDependencyPathApiImpl = exports.getPackageName = exports.isDependencyApiImpl = exports.isDependencyApi = exports.isDependencyApiOrApiImpl = exports.isDependencyOfType = exports.getDefaultPackageNameForModule = exports.getModuleSuffix = exports.getDefaultPackageNameForCamelCaseString = exports.splitCamelCaseString = exports.camelize = exports.isPublishedToNpm = void 0;
const clients_1 = require("./clients");
const PackagePath_1 = require("./PackagePath");
const gitCli_1 = require("./gitCli");
const lodash_1 = __importDefault(require("lodash"));
const Manifest_1 = require("./Manifest");
const ModuleTypes_1 = require("./ModuleTypes");
const log_1 = __importDefault(require("./log"));
const packageJsonFileUtils_1 = require("./packageJsonFileUtils");
const descriptors_1 = require("./descriptors");
const camelCase = require("lodash/camelCase");
const gitDirectoryRe = /.*\/(.*).git/;
async function isPublishedToNpm(pkg) {
    if (typeof pkg === 'string') {
        pkg = PackagePath_1.PackagePath.fromString(pkg);
    }
    let publishedVersions;
    try {
        publishedVersions = await clients_1.yarn.info(pkg, {
            field: 'versions',
        });
    }
    catch (e) {
        log_1.default.debug(e);
        return false;
    }
    if (publishedVersions) {
        const pkgVersion = PackagePath_1.PackagePath.fromString(pkg.toString()).version;
        return publishedVersions && pkgVersion
            ? publishedVersions.includes(pkgVersion)
            : true;
    }
    return false;
}
exports.isPublishedToNpm = isPublishedToNpm;
/**
 * Camelize name (parameter, property, method, etc)
 *
 * @param word string to be camelize
 * @param lowercaseFirstLetter lower case for first letter if set to true
 * @return camelized string
 */
function camelize(word, lowercaseFirstLetter = false) {
    word = camelCase(word);
    return (word &&
        word[0][lowercaseFirstLetter ? 'toLowerCase' : 'toUpperCase']() +
            word.substring(1));
}
exports.camelize = camelize;
/**
 * Split the camel case string
 *
 * @param camelCaseString
 * @returns {string}
 */
function splitCamelCaseString(camelCaseString) {
    return camelCaseString.split(/(?=[A-Z])/).map((token) => {
        return token.toLowerCase();
    });
}
exports.splitCamelCaseString = splitCamelCaseString;
function getDefaultPackageNameForCamelCaseString(moduleName, moduleType) {
    const splitArray = splitCamelCaseString(moduleName);
    switch (moduleType) {
        case ModuleTypes_1.API:
            return lodash_1.default.filter(splitArray, (token) => !['api'].includes(token)).join('-');
        case ModuleTypes_1.MINIAPP:
            return lodash_1.default.filter(splitArray, (token) => !['mini', 'app'].includes(token)).join('-');
        case ModuleTypes_1.JS_API_IMPL:
        case ModuleTypes_1.NATIVE_API_IMPL:
            return lodash_1.default.filter(splitArray, (token) => !['api', 'impl'].includes(token)).join('-');
        default:
            return splitArray.join('-');
    }
}
exports.getDefaultPackageNameForCamelCaseString = getDefaultPackageNameForCamelCaseString;
function getModuleSuffix(moduleType) {
    switch (moduleType) {
        case ModuleTypes_1.API:
            return 'api';
        case ModuleTypes_1.JS_API_IMPL:
            return 'api-impl-js';
        case ModuleTypes_1.MINIAPP:
            return 'miniapp';
        case ModuleTypes_1.NATIVE_API_IMPL:
            return 'api-impl-native';
        default:
            throw new Error(`Unsupported module type : ${moduleType}`);
    }
}
exports.getModuleSuffix = getModuleSuffix;
function getDefaultPackageNameForModule(moduleName, moduleType) {
    const basePackageName = getDefaultPackageNameForCamelCaseString(moduleName, moduleType);
    const suffix = getModuleSuffix(moduleType);
    return basePackageName.endsWith(suffix)
        ? basePackageName
        : `${basePackageName}-${suffix}`;
}
exports.getDefaultPackageNameForModule = getDefaultPackageNameForModule;
async function isDependencyOfType(dependency, types) {
    let a = [];
    a = a.concat(types);
    const ernField = await clients_1.yarn.info(dependency, {
        field: 'ern',
    });
    return a.indexOf(ernField === null || ernField === void 0 ? void 0 : ernField.moduleType) > -1;
}
exports.isDependencyOfType = isDependencyOfType;
async function isDependencyApiOrApiImpl(dependency) {
    return isDependencyOfType(dependency, [ModuleTypes_1.API, ModuleTypes_1.JS_API_IMPL, ModuleTypes_1.NATIVE_API_IMPL]);
}
exports.isDependencyApiOrApiImpl = isDependencyApiOrApiImpl;
async function isDependencyApi(dependency) {
    return isDependencyOfType(dependency, ModuleTypes_1.API);
}
exports.isDependencyApi = isDependencyApi;
async function isDependencyApiImpl(dependency) {
    return isDependencyOfType(dependency, [ModuleTypes_1.JS_API_IMPL, ModuleTypes_1.NATIVE_API_IMPL]);
}
exports.isDependencyApiImpl = isDependencyApiImpl;
async function getPackageName(pkg) {
    if (pkg.isGitPath) {
        throw new Error('getPackageName does not support git based package path yet');
    }
    return pkg.name;
}
exports.getPackageName = getPackageName;
async function isDependencyPathApiImpl(dependencyPath, type) {
    var _a;
    const modulesTypes = type ? [type] : [ModuleTypes_1.JS_API_IMPL, ModuleTypes_1.NATIVE_API_IMPL];
    const packageJson = await (0, packageJsonFileUtils_1.readPackageJson)(dependencyPath);
    return modulesTypes.indexOf((_a = packageJson.ern) === null || _a === void 0 ? void 0 : _a.moduleType) > -1;
}
exports.isDependencyPathApiImpl = isDependencyPathApiImpl;
async function isDependencyPathJsApiImpl(dependencyPath) {
    return isDependencyPathApiImpl(dependencyPath, ModuleTypes_1.JS_API_IMPL);
}
exports.isDependencyPathJsApiImpl = isDependencyPathJsApiImpl;
async function isDependencyPathNativeApiImpl(dependencyPath, type) {
    return isDependencyPathApiImpl(dependencyPath, ModuleTypes_1.NATIVE_API_IMPL);
}
exports.isDependencyPathNativeApiImpl = isDependencyPathNativeApiImpl;
/**
 * Version of react-native dependency in manifest
 */
async function reactNativeManifestVersion({ manifestId, } = {}) {
    const reactNativebasePathDependency = PackagePath_1.PackagePath.fromString('react-native');
    const reactNativeDependency = await Manifest_1.manifest.getNativeDependency(reactNativebasePathDependency, { manifestId });
    if (!reactNativeDependency) {
        throw new Error('Could not retrieve react native dependency from manifest');
    }
    return reactNativeDependency.version;
}
exports.reactNativeManifestVersion = reactNativeManifestVersion;
function isValidElectrodeNativeModuleName(name) {
    return /^[A-Z_][0-9A-Z_-]*$/i.test(name);
}
exports.isValidElectrodeNativeModuleName = isValidElectrodeNativeModuleName;
function logErrorAndExitProcess(e, code = 1) {
    if (e instanceof Error) {
        log_1.default.error(`An error occurred: ${e.message && e.message.trimRight()}`);
        log_1.default.debug(e.stack);
    }
    else if (e instanceof Object) {
        log_1.default.error(`An error occurred: ${JSON.stringify(e)}`);
    }
    else {
        log_1.default.error(`An error occurred: ${e}`);
    }
    process.exit(code);
}
exports.logErrorAndExitProcess = logErrorAndExitProcess;
function coerceToAppNameDescriptorArray(v) {
    return v instanceof Array
        ? v.map(coerceToAppNameDescriptor)
        : [coerceToAppNameDescriptor(v)];
}
exports.coerceToAppNameDescriptorArray = coerceToAppNameDescriptorArray;
function coerceToAppPlatformDescriptorArray(v) {
    return v instanceof Array
        ? v.map(coerceToAppPlatformDescriptor)
        : [coerceToAppPlatformDescriptor(v)];
}
exports.coerceToAppPlatformDescriptorArray = coerceToAppPlatformDescriptorArray;
function coerceToAnyAppDescriptorArray(v) {
    return v instanceof Array
        ? v.map(coerceToAnyAppDescriptor)
        : [coerceToAnyAppDescriptor(v)];
}
exports.coerceToAnyAppDescriptorArray = coerceToAnyAppDescriptorArray;
function coerceToAppVersionDescriptorArray(v) {
    return v instanceof Array
        ? v.map(coerceToAppVersionDescriptor)
        : [coerceToAppVersionDescriptor(v)];
}
exports.coerceToAppVersionDescriptorArray = coerceToAppVersionDescriptorArray;
function coerceToAppNameDescriptor(v) {
    return typeof v === 'string'
        ? descriptors_1.AppVersionDescriptor.fromString(v)
        : v.toAppNameDescriptor();
}
exports.coerceToAppNameDescriptor = coerceToAppNameDescriptor;
function coerceToAppPlatformDescriptor(v) {
    return typeof v === 'string'
        ? descriptors_1.AppPlatformDescriptor.fromString(v)
        : v.toAppPlatformDescriptor();
}
exports.coerceToAppPlatformDescriptor = coerceToAppPlatformDescriptor;
function coerceToAppVersionDescriptor(v) {
    return typeof v === 'string' ? descriptors_1.AppVersionDescriptor.fromString(v) : v;
}
exports.coerceToAppVersionDescriptor = coerceToAppVersionDescriptor;
function coerceToAnyAppDescriptor(v) {
    return typeof v === 'string' ? v.toAppDescriptor() : v;
}
exports.coerceToAnyAppDescriptor = coerceToAnyAppDescriptor;
function coerceToPackagePathArray(v) {
    return v instanceof Array
        ? v.map(coerceToPackagePath)
        : [coerceToPackagePath(v)];
}
exports.coerceToPackagePathArray = coerceToPackagePathArray;
function coerceToPackagePath(v) {
    return typeof v === 'string' ? PackagePath_1.PackagePath.fromString(v) : v;
}
exports.coerceToPackagePath = coerceToPackagePath;
const gitRefBranch = (branch) => `refs/heads/${branch}`;
const gitRefTag = (tag) => `refs/tags/${tag}`;
const gitShaLength = 40;
async function isGitBranch(p) {
    if (!p.isGitPath) {
        return false;
    }
    if (p.isGitPath && !p.version) {
        return true;
    }
    const heads = await (0, gitCli_1.gitCli)().listRemote(['--heads', p.basePath]);
    return heads.includes(gitRefBranch(p.version));
}
exports.isGitBranch = isGitBranch;
async function isGitTag(p) {
    if (!p.isGitPath || !p.version) {
        return false;
    }
    const tags = await (0, gitCli_1.gitCli)().listRemote(['--tags', p.basePath]);
    return tags.includes(gitRefTag(p.version));
}
exports.isGitTag = isGitTag;
async function getCommitShaOfGitBranchOrTag(p) {
    if (!p.isGitPath) {
        throw new Error(`${p} is not a git path`);
    }
    if (!p.version) {
        throw new Error(`${p} does not include a branch or tag`);
    }
    const result = await (0, gitCli_1.gitCli)().listRemote([p.basePath, p.version]);
    if (!result || result === '') {
        throw new Error(`${p.version} branch or tag not found in ${p.basePath}`);
    }
    return result.substring(0, gitShaLength);
}
exports.getCommitShaOfGitBranchOrTag = getCommitShaOfGitBranchOrTag;
async function getCommitShaOfGitPackage(p) {
    if (!p.isGitPath) {
        throw new Error(`${p} is not a git path`);
    }
    if ((await isGitBranch(p)) || (await isGitTag(p))) {
        return getCommitShaOfGitBranchOrTag(p);
    }
    return p.version;
}
exports.getCommitShaOfGitPackage = getCommitShaOfGitPackage;
//# sourceMappingURL=utils.js.map